;+
; Project     : SOHO-CDS
;
; Name        : GET_XWIN
;
; Purpose     : get a free X-window
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : get_xwin,index
;
; Examples    :
;
; Inputs      : INDEX - window index to get
;               If input window INDEX is available then corresponding window
;               will become active, otherwise a new window is opened
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: 
;
; Keywords    : XSIZE,YSIZE = window size [def = 512,512]
;               RETAIN = retain backing store [def = 2]
;               XPOS,YPOS = window position [def = 0,0]
;               TITLE = window title [def = '']
;               FREE  = get next free window
;               NOSHOW = don't call WSHOW
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 October 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-
;+

pro get_xwin,index,xsize=xsize,ysize=ysize,free=free,_extra=extra,$
             ypos=ypos,xpos=xpos,retain=retain,title=title,draw=draw,$
             noshow=noshow

if (!d.name ne 'X') and (!d.name ne 'WIN') then return
if n_elements(retain) eq 0 then retain=2
if n_elements(xpos) eq 0 then xpos=0
if n_elements(ypos) eq 0 then ypos=0
if (n_elements(xsize) eq 0) and (n_elements(ysize) ne 0) then xsize=ysize(0)
if (n_elements(ysize) eq 0) and (n_elements(xsize) ne 0) then ysize=xsize(0)
if (n_elements(ysize) eq 0) and (n_elements(xsize) eq 0) then begin
 xsize=512 & ysize=512
endif
if n_elements(title) eq 0 then title=''

dt=datatype(index,2)
if (dt gt 5) or (dt eq 0) then index=-1
avail=0
device,window=ow
owx=where(ow eq 1)
find=where(float(index) eq float(owx),count)
get_new=0
if keyword_set(free) then begin
 get_new=1 & index=-1
endif else begin
 if count gt 0 then begin
  index=(owx(find))(0)
  if index gt -1 then begin
   if (!d.x_size ne xsize) or (!d.y_size ne ysize) then begin
    get_new=1 & index=-1
   endif else wset,index
  endif else get_new=1
 endif else get_new=1
endelse

error=0
catch,error
if error ne 0 then begin
 message,'closed window',/cont
 get_new=1 & index=-1
endif

if get_new then begin
 if index gt -1 then begin
  window,index,xsize=xsize,ysize=ysize,xpos=xpos,retain=retain,title=title,$
   ypos=ypos,_extra=extra
 endif else begin
  window,/free,xsize=xsize,ysize=ysize,xpos=xpos,retain=retain,$
  title=title,ypos=ypos,_extra=extra
 endelse
endif

index=!d.window
message,'current window set to: '+string(index),/cont
if (get_new) and title eq '' then wtitle,index,'IDL '+trim(string(index))
if (1-keyword_set(draw)) and (1-keyword_set(noshow)) then wshow,index 
wset,index

return & end

