	PRO PLOTBAR,X,Y,TITLE=TITLE,XTITLE=XTITLE,YTITLE=YTITLE,	$
		BCOLOR=BCOLOR, XTYPE=XTYPE,YTYPE=YTYPE,XRANGE=XRANGE,	$
		YRANGE=YRANGE
;+
; Project     :	SOHO - CDS
;
; Name        :	PLOTBAR
;
; Purpose     :	Plot a bar graph filled in with a pattern.
;
; Category    :	Class3, Graphics
;
; Explanation :	
;
; Syntax      :	PLOTBAR, [ X ,]  Y
;
; Examples    :	
;
; Inputs      :	Y = array of values along the Y axis.
;
; Opt. Inputs :	X = optional array of values along the X axis.  Y(i) is plotted
;		    from X(i) to X(i+1), with the last point being plotted over
;		    a distance equal to its predecessor.  The procedure tests
;		    whether or not the second parameter passed is a vector to
;		    decide if X was passed.  If not passed, then INDGEN(Y) is
;		    assumed.
;
;		If !BCOLOR is nonzero, then it controls the color of the filled
;		in region.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	TITLE	= Main plot title, default is !P.TITLE.
;		XTITLE	= X axis title, default is !X.TITLE.
;		YTITLE	= Y axis title, default is !Y.TITLE.
;		XTYPE	= If set, then X axis is logarithmic.
;		YTYPE	= If set, then Y axis is logarithmic.
;		BCOLOR	= Color to use instead of !BCOLOR for bar interiors.
;		XRANGE	= Range of data values in the X direction.
;		YRANGE	= Range of data values in the Y direction.
;
; Calls       :	POLYFILL
;
; Common      :	None.
;
; Restrictions:	The variables must not be of type string.
;
; Side effects:	None.
;
; Prev. Hist. :	
;	William Thompson	Applied Research Corporation
;	July, 1986		8201 Corporate Drive
;				Landover, MD  20785
;
;	William Thompson, December 1991, added keywords XTYPE and YTYPE.
;	W.T.T., May 1992, added BCOLOR keyword.
;	William Thompson, May 1992, added keywords XRANGE and YRANGE.
;	William Thompson, November 1992, modified to be compatible with
;		!P.MULTI.
;	William Thompson, January 1993, modified to use modern system parameter
;		names.
;	William Thompson, 30 April 1993, fixed bug with !P.MULTI.
;
; History     :	Version 1, 22-Jan-1996, William Thompson, GSFC
;			Incorporated into CDS library
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR,2
;
	PSYM = !PSYM
	LINETYPE = !LINETYPE
	C = !C
;
	IF N_PARAMS(0) EQ 1 THEN BEGIN			;Only Y array passed.
		XX = FINDGEN(N_ELEMENTS(X))
		YY = X
	END ELSE IF N_PARAMS(0) EQ 2 THEN BEGIN
		XX = X
		YY = Y
	END ELSE BEGIN
		PRINT,'*** PLOTBAR must be called with 1-2 parameters:'
		PRINT,'                   [ X ,]  Y'
		RETURN
	ENDELSE
;
	M = N_ELEMENTS(XX) < N_ELEMENTS(YY)
	IF M LE 1 THEN BEGIN
		PRINT,'*** Not enough points to plot, routine PLOTBAR.'
		RETURN
	ENDIF
;
;  Generate the arrays used in plotting.  Fill in X array so that last
;  bar has same width as previous one.
;
	I = FINDGEN(2*M)/2
	X_LAST = 2*XX(M-1) - XX(M-2)
	XX = [XX(I),X_LAST,X_LAST]
	YY = [0,YY(I),0]
;
	!PSYM=0					;Save system variables.
	!LINETYPE = 0
	!C = 0
;
;  Do the initial plot, keeping track of the setting of !P.MULTI.
;
	PMULTI = !P.MULTI
	COMMAND = "PLOT,XX,YY"
	IF N_ELEMENTS(XTITLE) EQ 1 THEN COMMAND = COMMAND + ",XTITLE=XTITLE"
	IF N_ELEMENTS(YTITLE) EQ 1 THEN COMMAND = COMMAND + ",YTITLE=YTITLE"
	IF N_ELEMENTS(TITLE)  EQ 1 THEN COMMAND = COMMAND + ",TITLE=TITLE"
	IF N_ELEMENTS(XTYPE)  EQ 1 THEN COMMAND = COMMAND + ",XTYPE=XTYPE"
	IF N_ELEMENTS(YTYPE)  EQ 1 THEN COMMAND = COMMAND + ",YTYPE=YTYPE"
	IF N_ELEMENTS(XRANGE) EQ 2 THEN COMMAND = COMMAND + ",XRANGE=XRANGE"
	IF N_ELEMENTS(YRANGE) EQ 2 THEN COMMAND = COMMAND + ",YRANGE=YRANGE"
	TEST = EXECUTE(COMMAND)
;
	XX = !CXMIN > XX < !CXMAX
	YY = !CYMIN > YY < !CYMAX
	IF N_ELEMENTS(BCOLOR) NE 1 THEN BCOLOR = !BCOLOR
	IF BCOLOR EQ 0 THEN COLOR=!COLOR ELSE COLOR=BCOLOR
;
;  Fill in histogram and overplot boundary to restore color.
;
	POLYFILL,XX,YY,/DATA,COLOR=COLOR
	NOERAS = !NOERAS
	!NOERAS = -1
	XTITLE = !X.TITLE  &  !X.TITLE = ''
	YTITLE = !Y.TITLE  &  !Y.TITLE = ''
	PTITLE = !P.TITLE  &  !P.TITLE = ''
	!P.MULTI = PMULTI
;
	COMMAND = "PLOT,XX,YY"
	IF N_ELEMENTS(XTYPE)  EQ 1 THEN COMMAND = COMMAND + ",XTYPE=XTYPE"
	IF N_ELEMENTS(YTYPE)  EQ 1 THEN COMMAND = COMMAND + ",YTYPE=YTYPE"
	IF N_ELEMENTS(XRANGE) EQ 2 THEN COMMAND = COMMAND + ",XRANGE=XRANGE"
	IF N_ELEMENTS(YRANGE) EQ 2 THEN COMMAND = COMMAND + ",YRANGE=YRANGE"
	TEST = EXECUTE(COMMAND)
;
;  Modify !P.MULTI.  Normally, this would happen automatically, but this has
;  been disabled for this routine.
;
	K = !P.MULTI(0)
	NX = !P.MULTI(1) > 1
	NY = !P.MULTI(2) > 1
	IF (K LE 0) OR (K GT NX*NY) THEN K = NX*NY
	!P.MULTI(0) = (K - 1) MOD (NX*NY)
;
	!NOERAS = NOERAS
	!X.TITLE = XTITLE  &  !Y.TITLE = YTITLE  &  !P.TITLE = PTITLE
	OPLOT,[!CXMIN,!CXMAX],[0,0]			;Plot zero line.
;
	!PSYM = PSYM
	!LINETYPE = LINETYPE
	!C = C
;
	RETURN
	END
