;+
; Project     : SOHO - CDS
;
; Name        : JSMOVIE
;
; Purpose     : Make a Javascript movie HTML file for a series of images
;               (GIF, JPEG, etc)
;
; Category    : Imaging
;
; Explanation : Program reads a template HTML file 
;               and replaces key parameters with user specified values
;
; Syntax      : IDL> jsmovie,file,images
;
; Inputs      : FILE = filename to which Javascript code will be written
;                      [def=jsmovie_new.html]
;               IMAGES = array of image filenames (e.g. ['frame0.gif','frame1.gif..])
;
; Opt. Inputs : None
;
; Outputs     : None other than Javascript HTML file
;
; Opt. Outputs: None
;
; Keywords    : DELAY = millisecond delay between frames [def=250]
;               INCREMENT  = % increment for speed control [def= 10]
;               URL = optional URL path to images [def = relative to current dir]
;               SIZE = optional pixel size of images as 2-d vector [nx,ny]
;               TITLE = optional HTML title
;               BORDER, CELL = optional border width & cellpadding for image
;                              frame (def = 10 & 8 pixels)
;               TEMPLATE = template file name [def= "jsmovie_temp.html"]
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Version 1,  4-July-1997,  D.M. Zarro.  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro jsmovie,file,images,url=url,delay=delay,increment=increment,$
      template=template,size=gsize,title=title,force=force,border=border,cell=cell

common jsmovie,template_name

if datatype(template) ne 'STR' then template_file='jsmovie_temp.html' else $
 template_file=template

if (datatype(template_name) ne '') or keyword_set(force) then $
 template_name=find_with_def(template_file,!path)

if trim(template_name) eq '' then begin
 message,'template file "'+template_file+'" not found',/cont
 return
endif

if (not exist(images)) then begin
 message,'Syntax -> jsmovie,file,images',/cont
 return
endif

;-- check output file

if datatype(file) ne 'STR' then file='jsmovie_new.html'
break_file,file,fdsk,fdir,fname,fext
floc=trim(fdsk+fdir)
cd,curr=cdir
if floc eq '' then floc=cdir
if not chk_dir(floc) then begin
 message,'Invalid output directory name: '+floc,/cont
 return
endif
if not test_open(floc) then begin
 message,'No write access to directory: '+floc,/cont
 return
endif
if fext eq '' then fext='.html'
fname=concat_dir(floc,fname+fext)

;-- check keywords

if not exist(border) then border=10
if not exist(cell) then cell=8
if not exist(delay) then delay=250.
if not exist(increment) then increment=50.
nx=0 & ny=0
if exist(gsize) then begin
 nx=gsize(0) & ny=nx
 if n_elements(gsize) gt 1 then ny=gsize(1)
endif
if datatype(title) eq 'STR' then stitle=title

;-- read template_name

jsfile=rd_ascii(template_name)
if n_elements(jsfile) eq 1 then begin
 message,'invalid template file: '+template_name,/cont
 return
endif

;-- update titles

if datatype(stitle) eq 'STR' then begin

 jtitle=grep('<TITLE>',jsfile,index=index)
 if index(0) gt -1 then begin
  if trim(stitle) ne '' then jtitle='<TITLE>'+title+'</TITLE>' else jtitle=''
  jsfile(index(0))=jtitle
 endif

 jtitle=grep('<H1>',jsfile,index=index)
 if index(0) gt -1 then begin
  if trim(title) ne '' then jtitle='<H1>'+title+'</H1>' else jtitle=''
  jsfile(index(0))=jtitle
 endif
endif

;-- update TABLE parameters

jtable=grep('<TABLE',jsfile,index=index)
if index(0) gt -1 then begin
 jtable='<TABLE BORDER="'+trim(string(border))+'" CELLPADDING="'+$
          trim(string(cell))+'">'
 jsfile(index(0))=jtable
endif

;-- update GIF file path

if datatype(url) ne 'STR' then surl='' else surl=trim(url)
nurl=n_elements(surl)
if surl(0) ne '' then begin
 jimax=grep('var url_path',jsfile,index=index)
 if index(0) gt -1 then begin
  jsfile(index(0))='var url_path = "'+surl(0)+'";'
 endif
endif 

jsrc=grep('<img',jsfile,index=index)
if index(0) gt -1 then begin
 jsrc='<img src = "&{url_path};"'+'+"/'+images(0)+'" NAME=animation ALT="FRAME"'

 if (nx gt 0) and (ny gt 0) then begin
  jsrc=jsrc+' width='+trim(string(nx))+' height='+trim(string(ny))
 endif
 jsrc=jsrc+'>'
 jsfile(index(0))=jsrc
endif

;-- update variable parameters

imax=n_elements(images)
ok=where(trim(images) ne '',imax)
if imax eq 0 then begin
 message,'found all blank GIF file names',/cont
 return
endif
images=images(ok)
 
inc=trim(string(1.+increment/100.,'(f5.2)'))
imax=trim(string(imax))
sdelay=trim(string(long(delay)))
control_par='var imax = '+imax+', inc = '+inc+', delay = '+sdelay+';'

jimax=grep('var imax',jsfile,index=index)
if index(0) gt -1 then begin
 jsfile(index(0))=control_par
endif

;-- update images array

temp=strarr(imax,3)
for i=0,imax-1 do begin
 ifile='url_path+"/'+images(i)+'"

 istring='images['+trim(string(i))+']'
 temp(i,0)=istring+' = new Image();'
; temp(i,1)=istring+'.onabort = image_abort;'
; temp(i,2)=istring+'.onerror = image_error("Error loading: "+'+ifile+');'
 temp(i,1)=istring+'.onload = count_images;'
 temp(i,2)=istring+'.src = '+ifile+';'
endfor

jimg=grep('images[0] = ',jsfile,index=index)

if index(0) gt 0 then begin
 jsfile(index(0))=''
 pre=jsfile(0:index(0))
 post=jsfile(index(0)+1:*)
 jsfile=pre
 for k=0,imax-1 do $
  jsfile=[jsfile,temp(k,0),temp(k,1),temp(k,2)]
 jsfile=[jsfile,post]
endif

file_append,fname,jsfile,/new

message,'wrote Javascript file: '+fname,/cont

return & end

