;+
; Project     : SOHO-CDS
;
; Name        : MK_MPEG
;
; Purpose     : Make MPEG file from list of GIF files
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : IDL> mk_mpeg,file
;
; Examples    :
;
; Inputs      : FILENAME = MPEG filename
;             : LIS_GIF = list of GIF filenames to convert
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: None
;
; Keywords    : PATH = path to MPEG maker programs
;
; History     : Written 22 December 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

pro mk_mpeg,filename,lis_gif,path=path

if (datatype(filename) ne 'STR') or (datatype(lis_gif) ne 'STR') then begin
 message,'Syntax: mk_mpeg,mpeg_file,gif_list',/cont
 return
endif

;-- check inputs

if datatype(lis_gif) ne 'STR' then begin
 message,'Enter filename with list of GIF files to convert',/cont
 return
endif

;-- look for MPEG-makers

if datatype(path) ne 'STR' then begin
 cpath=''
 espawn,'which mpeg_encode',out
 out=out(0)
 if trim(out) ne '' then not_found=(strpos(out,'not found') gt -1) else not_found=1
endif else begin
 cpath=path
 clook=loc_file(concat_dir(cpath,'mpeg_encode'),count=count)
 not_found=count eq 0
endelse

if not_found then begin
 message,'MPEG_ENCODE not found',/cont
 return
endif

if datatype(path) ne 'STR' then begin
 cpath=''
 espawn,'which giftoppm',out
 out=out(0)
 if trim(out) ne '' then not_found=(strpos(out,'not found') gt -1) else not_found=1
endif else begin
 cpath=path
 clook=loc_file(concat_dir(cpath,'giftoppm'),count=count)
 not_found=count eq 0
endelse
 
if not_found then begin
 message,'GIFTOPPM not found',/cont
 return
endif

if datatype(filename) ne 'STR' then begin
 message,'Enter MPEG output filename',/cont
 return
endif

;-- make sure MPEG_ENCODE can output MPEG file to required directory

cd,cur=cur_dir
break_file,filename,fdsk,fdir,file,ext
out_dir=trim(fdsk+fdir)
if out_dir eq '' then out_dir=cur_dir
ok=test_open(out_dir,/write)
if not ok then begin
 message,'Cannot write MPEG file to: '+out_dir
 return
endif
olook=loc_file(out_dir,count=count)
if count eq 0 then begin
 message,'Cannot locate output directory for writing MPEG file',/cont
 return
endif
out_dir=olook(0)
ofile=concat_dir(out_dir,trim(file+ext))

;-- assume GIF list in same location as GIF files

dgif=cur_dir
if n_elements(lis_gif) gt 1 then gifs=lis_gif else begin
 break_file,lis_gif,dsk,ddir
 if trim(dsk+ddir) eq '' then lgif=concat_dir(dgif,lis_gif) else lgif=lis_gif
 glook=loc_file(lgif,count=count)
 if count eq 0 then begin
  message,'Cannot locate list of GIF file names: '+lgif,/cont
  return
 endif
 gifs=rd_ascii(lgif,err=err)
 if err then begin
  message,'Error reading: '+lgif,/cont
  return
 endif
endelse

;-- check existence of GIF directory

break_file,gifs,dsk,ddir,fgifs,ext
bgif=trim(dsk(0)+ddir(0))
if bgif ne '' then dgif=bgif
clook=loc_file(dgif,count=count)
if count eq 0 then begin
 message,'Cannot locate GIF directory: '+dgif,/cont
 return
endif
dgif=clook(0)
       
;-- check existence of individual GIF files

nframes=n_elements(fgifs)
fgifs=trim(fgifs+ext)
igifs=concat_dir(dgif,fgifs)
for i=0,nframes-1 do begin
 igif=loc_file(igifs(i),count=count)
 if count gt 0 then begin
  if valid_gif(igif(0)) then begin
   if not exist(ogifs) then ogifs=fgifs(i) else ogifs=[ogifs,fgifs(i)]
  endif
 endif
endfor
 
if not exist(ogifs) then begin
 message,'Cannot find any GIF files in list',/cont
 return
endif

nframes=n_elements(fgifs)

;
; Open the mpeg parameter file for writing.  Store logical unit number in
; MPRM.
;

           PARAMFILE=concat_dir(out_dir,'params.mpeg')
	   OPENW, MPRM, PARAMFILE, /GET_LUN
           
;
; Write out the mpeg parameter file.
;
	   PRINTF, MPRM, 'PATTERN          IBBBBBBBBBBP'
	   PRINTF, MPRM, 'OUTPUT           ' + ofile
	   PRINTF, MPRM, 'GOP_SIZE 12'
	   PRINTF, MPRM, 'SLICES_PER_FRAME 5'
	   PRINTF, MPRM, 'BASE_FILE_FORMAT PPM'
	   PRINTF, MPRM, 'INPUT_CONVERT giftoppm *'
	   PRINTF, MPRM, 'INPUT_DIR        ' + DGIF
	   PRINTF, MPRM, 'INPUT'
           for i=0,nframes-1 do printf,mprm,fgifs(i)
;
;	   PRINTF, MPRM, 'frame*.gif [' + trim(STRING(istart)) + '-' +	$
;		trim(STRING(iend)) + ']'
	   PRINTF, MPRM, 'END_INPUT'
	   PRINTF, MPRM, 'PIXEL            FULL'
	   PRINTF, MPRM, 'RANGE            5'
	   PRINTF, MPRM, 'PSEARCH_ALG      LOGARITHMIC'
	   PRINTF, MPRM, 'BSEARCH_ALG      SIMPLE'
	   PRINTF, MPRM, 'IQSCALE          6'
	   PRINTF, MPRM, 'PQSCALE          6'
	   PRINTF, MPRM, 'BQSCALE          6'
	   PRINTF, MPRM, 'REFERENCE_FRAME  ORIGINAL'
	   PRINTF, MPRM, 'FORCE_ENCODE_LAST_FRAME'
;
; Close the mpeg parameter file.
;
	   FREE_LUN, MPRM

;
; spawn a shell to process the mpeg_encode command
;
	   SPAWN, trim(concat_dir(cpath,'mpeg_encode'))+' '+ PARAMFILE

           rm_file,paramfile

           return & end
