;+
; Project     : SOHO-CDS
;
; Name        : DROT_MAP
;
; Purpose     : differentially rotate image contained within structure created by MAKE_MAP
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : rmap=drot_map(map,duration)
;
; Examples    :
;
; Inputs      : MAP = map structure
;               DURATION = hours to rotate by
;
; Opt. Inputs : None
;
; Outputs     : RMAP = map with rotated coordinates
;
; Opt. Outputs: None
;
; Keywords    : SECONDS = duration units in seconds
;               TIME = time to rotate image to (can be a MAP or INDEX stc)
;               NOREMAP = don't remap image data (just compute rotated coords)
;               DAYS = specify duration units in days
;               TAG_ID = tag name or index to rotate
;               CENTER = [rxc,ryc] = new rotation center of output image (if rolling)
;               TRANS = [xs,ys] = translation of output image [+N, +W]
;               SPACE = [dx,dy] = spacing of output image
;               OUTSIZE  = [nx,ny] = size of output image
;               ROLL  = new roll of output image
;               NOLIMB = exclude off limb points
;               MISSING = values to set off limb points
;               NCENTER = [xc,yc] = new center of output image coords
;               REF_MAP = reference image to map to. 
;                         If entered, then TIME, SPACING, ROLL are inherited
;               EARTH = set to convert to EARTH-view (not yet implemented)
;               SOHO = set to convert to SOHO-view (not yet implemented)
;               FAST = set when one wants to solely shift the entire FOV 
;                      without correcting for the latitudinal dependence of 
;                      the solar differential rotation function.
;                      USE WITH CARE! - results are only meaningful for small
;                      rotation intervals of a few hours for images within
;                      10-20 degrees of the equator.
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 November 1997, D. Zarro, SAC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function drot_map,map,duration,seconds=seconds,days=days,_extra=extra,$
                  time=time,noremap=noremap,err=err,$
                  verbose=verbose,missing=missing,trans=trans,$
                  tag_id=tag_id,space=space,roll=roll,fast=fast,$
                  center=center,ncenter=ncenter,nolimb=nolimb,$
                  ref_map=ref_map,earth=earth,soho=soho,outsize=outsize

if not valid_map(map) then begin
 pr_syntax,'rmap=drot_map(map,duration,[time=time])'
 return,-1
endif

;-- check keywords

nolimb=keyword_set(nolimb)
to_earth=keyword_set(earth)
to_soho=keyword_set(soho)
if to_earth and to_soho then begin
 message,'/EARTH and /SOHO are mutually exclusive',/cont
 return,map
endif

verbose=keyword_set(verbose)
units='(hours)'
days=keyword_set(days)
seconds=keyword_set(seconds)
if seconds then units='(secs)'
if days then units='(days)'

;-- if REF_MAP entered then use it's TIME, SPACING, CENTER, ROLL, VIEW, and
;    DIMENSIONS

if valid_map(ref_map) then begin
 time=anytim2utc(ref_map(0).time)
 unpack_map,ref_map(0),xc=xc,yc=yc,dx=dx,dy=dy,$
  nx=nx,ny=ny,rcenter=droll_center,roll=droll
 dspace=[dx,dy]
 dcenter=[xc,yc]
 dsize=[nx,ny]
 dview=get_map_prop(ref_map,/soho) > 0
endif

;-- get solar rotation duration

cur_time=anytim2tai(map.time)
if not exist(duration) then begin
 if valid_map(time) then begin
  if tag_exist(time,'time') then tend=anytim2tai(time.time)
 endif else if exist(time) then tend=anytim2tai(anytim(time,/mjd))
 if exist(tend) then dtime=(tend-cur_time)/3600.
 if not exist(dtime) then begin
  repeat begin
   dtime='' & read,'* enter duration '+units+' to rotate by : ',dtime
  endrep until dtime ne ''
  dtime=float(dtime)
 endif
endif else dtime=duration

case units of
 '(days)': dtime=dtime*3600.*24.
 '(hours)': dtime=dtime*3600.
 else:do_nothing=1
endcase

;-- FAST option

remap=(1-keyword_set(noremap)) and (1-keyword_set(fast))
 
;-- translate after rotation?

xs=0. & ys=0.
do_trans=n_elements(trans) eq 2
if do_trans then begin
 xs=trans(0) & ys=trans(1)
 do_trans=(xs ne 0.) and (ys ne 0.)
endif

tags=tag_names(map) & ntags=n_elements(tags) & nmp=n_elements(map)
have_rtime=tag_exist(map,'rtime')
have_roll_center=tag_exist(map,'roll_center')
have_view=tag_exist(map,'soho')
ntime=n_elements(dtime)

for i=0,nmp-1 do begin
 unpack_map,map(i),data,xp,yp,err=err,dx=dx,dy=dy,tag_id=tag_id,$
          nx=nx,ny=ny,roll=curr_roll,xc=xc,yc=yc,rcenter=curr_rcenter
 if not exist(tag_id) then tag_no=0 else tag_no=get_tag_index(map(i),tag_id)

;-- check if remapping views (view=1 maps to SOHO-view, 0 maps to Earth)
 
 curr_view=get_map_prop(map(i),/soho) > 0.
 new_view=curr_view
 if exist(dview) then new_view=dview
 if to_earth then new_view=0
 if to_soho then new_view=1
 do_view=(new_view ne curr_view)

;-- check if already solar rotated

 cdur=dtime(i < (ntime-1))
 dprint,'% duration (sec): ',cdur
 new_time=cur_time(i)+cdur
 if have_rtime then $
  already_rotated=(new_time eq anytim2tai(map(i).rtime)) else $
   already_rotated=(new_time eq anytim2tai(map(i).time)) 

;-- check if rolling

 have_roll=(curr_roll mod 360.0) gt 0.
 new_roll=curr_roll
 if exist(droll) then new_roll=droll
 if exist(roll) then new_roll=float(roll)
 roll_diff=abs(curr_roll-new_roll)
 do_roll=(roll_diff mod 360.) gt 0.

 if verbose and (i eq 0) then begin
  if have_roll then $
   message,'correcting for '+trim(string(curr_roll))+' degree roll',/cont
  if do_roll then $
   message,'applying new '+trim(string(new_roll))+' degree roll',/cont
 endif

;-- check if new roll center

 new_rcenter=curr_rcenter
 if n_elements(droll_center) eq 2 then new_rcenter=droll_center
 if n_elements(center) eq 2 then new_rcenter=float(center)
 do_rcenter=(do_roll or have_roll) and ((new_rcenter(0) ne curr_rcenter(0)) or $
            (new_rcenter(1) ne curr_rcenter(1)))

;-- check if recentering 
;  (if an array of images, then track relative to first)

 do_center=0
 if (i eq 0) and (nmp gt 1) then new_center=[xc,yc]
 if n_elements(dcenter) eq 2 then new_center=dcenter
 if n_elements(ncenter) eq 2 then new_center=ncenter
 if exist(new_center) then $
  do_center=(new_center(0) ne xc) or (new_center(1) ne yc)
  
;-- check if rebinning

 if i eq 0 then new_space=[dx,dy]
 if n_elements(dspace) eq 2 then new_space=dspace
 if n_elements(space) eq 2 then new_space=space
 do_rebin=(new_space(0) ne dx) or (new_space(1) ne dy)

;-- check if resizing

 if i eq 0 then new_size=[nx,ny]
 if n_elements(dsize) eq 2 then new_size=dsize
 if n_elements(outsize) eq 2 then new_size=outsize
 do_resize=(new_size(0) ne nx) or (new_size(1) ne ny)

 dprint,'% already_rotated,do_rebin,do_roll,do_center,do_trans,do_view,do_rcenter, do_resize: ',$
  already_rotated,do_rebin,do_roll,do_center,do_trans,do_view,do_rcenter,do_resize

 onx=new_size(0) & ony=new_size(1)
 nmap=map(i)
 if not have_roll then nmap=add_tag(nmap,0.,'roll',index='id')
 if not have_roll_center then nmap=add_tag(nmap,[xc,yc],'roll_center',index='roll')
 if not have_rtime then nmap=add_tag(nmap,'','rtime',index='time')
 if not have_view then nmap=add_tag(nmap,curr_view,'soho')

 nmap.rtime=anytim2utc(new_time,/vms)
 if do_roll then begin
  nmap.roll=new_roll
  nmap.roll_center=new_rcenter
 endif else begin
  if have_roll then begin
   nmap.roll=curr_roll
   nmap.roll_center=curr_rcenter
  endif
 endelse

 if already_rotated and $
  (not do_rcenter) and (not do_trans) and $
  (not do_rebin) and (not do_roll) and $
  (not do_center) and (not do_view) and (not do_resize) then begin
  dprint,'% already rotated'
  goto,done
 endif

;-- correct current roll before solar rotating

 xr=xp & yr=yp
 if have_roll then rot_cord,xr,yr,-curr_roll,xr,yr,center=curr_rcenter

;-- solar rotate here

 xr=reform(temporary(xr),nx*ny)
 yr=reform(temporary(yr),nx*ny)
 rcor=rot_xy(xr,yr,tstart=cur_time(i),tend=new_time,soho=curr_view,/keep,$
             radius=radius,index=on_disk)
 rad=radius*60.
 sz=size(rcor)
 if sz(2) eq 2 then rcor=transpose(temporary(rcor))
 
;-- update pixel arrays (shift and re-roll) 

 xr=reform(rcor(0,*),nx,ny)+xs
 yr=reform(rcor(1,*),nx,ny)+ys
  
 if do_roll then rot_cord,xr,yr,new_roll,xr,yr,center=new_rcenter else $
  if have_roll then rot_cord,xr,yr,curr_roll,xr,yr,center=curr_rcenter

;-- put over limb points onto limb at (radius,0)

 if not already_rotated then begin
  rot_pos=sqrt(xr^2+yr^2)
  rover_limb=where( rot_pos gt rad,rover_count)
  dprint,'% DROT_MAP: # of points rotated over limb =',trim(string(rover_count))
  if rover_count eq nx*ny then begin
   err='All points rotated over limb'
   message,err,/cont
  endif else begin
   if (rover_count gt 0) then begin
    xr(rover_limb)=rad
    yr(rover_limb)=0.
   endif
  endelse
 endif

;-- map view

 if do_view then begin
  soho_cord,xr,yr,new_time,xr,yr,inverse=curr_view
  nmap.soho=new_view
 endif

;-- update map properties 
;   (if not remapping pixels, save in old format to preserve irregular 
;    coordinates)
 
 if not remap then nmap=mk_map_old(nmap)
 nmap=repack_map(nmap,xr,yr)

;-- remap image

 if remap then begin

  if not exist(missing) then missing=0.

;-- first make a regularized grid

  if not already_rotated then begin
   on_disk=where(rot_pos le rad,cnt)
   grid_xy,xr(on_disk),yr(on_disk),gx,gy,$
           space=new_space,center=new_center,size=new_size

   rot_pos=sqrt(gx^2+gy^2)
   rover_limb=where( rot_pos gt rad,rover_count)
  endif else begin
   grid_xy,xr,yr,gx,gy,space=new_space,center=new_center,size=new_size
  endelse

;-- map grid points back to find where each point came from

  xr=gx
  yr=gy

;-- remap view

  if do_view then soho_cord,xr,yr,new_time,xr,yr,inverse=new_view

;-- roll-back if necessary

  if do_roll then rot_cord,xr,yr,-new_roll,xr,yr,center=new_rcenter else $
   if have_roll then rot_cord,xr,yr,-curr_roll,xr,yr,center=curr_rcenter

;-- shift back

  xr=temporary(xr)-xs
  yr=temporary(yr)-ys

;-- solar rotate backward

  xr=reform(temporary(xr),onx*ony)
  yr=reform(temporary(yr),onx*ony)
  rcor=rot_xy(xr,yr,tstart=new_time,tend=cur_time(i),soho=soho,/keep)
  sz=size(rcor)
  if sz(2) eq 2 then rcor=transpose(temporary(rcor))
  xr=reform(rcor(0,*),onx,ony)
  yr=reform(rcor(1,*),onx,ony)

;-- roll back to initial roll

  if have_roll then rot_cord,xr,yr,curr_roll,xr,yr,center=curr_rcenter

;-- don't solar rotate overlimb points

  if not already_rotated then begin
   over_limb=where(sqrt(xp^2+yp^2) gt rad,over_count)
   if over_count eq nx*ny then begin
    err='No points on disk'
    message,err,/cont
   endif
   if nolimb and (over_count gt 0) then data(over_limb)=missing
  endif
 
;-- remap here

  nmap=grid_map(nmap,onx,ony)
  nmap.(tag_no)=interp2d(data,xp,yp,xr,yr,_extra=extra,missing=missing)
 
;-- update map structure parameters

  nmap=repack_map(nmap,gx,gy)

;-- set rotated over limb points to MISSING

  if not already_rotated then begin
   if nolimb then begin
    if (rover_count gt 0) then nmap.(tag_no)(rover_limb)=missing 
    dmin=min(nmap.(tag_no),max=dmax)
    if (dmin eq dmax) and ((dmin eq 0) or (dmin eq missing)) then begin
     err='Warning zero data points in field of view'
     message,err,/cont
    endif
   endif
  endif

 endif
done:

 if not exist(rmap) then rmap=temporary(nmap) else begin
  c1=valid_map(rmap,old=rold)
  c2=valid_map(nmap,old=nold)
  if nold ne rold then begin
   if rold then nmap=mk_map_old(nmap) else nmap=mk_map_new(nmap)
  endif
  rmap=merge_struct(temporary(rmap),temporary(nmap))
 endelse
endfor

delvarx,xp,yp,xr,yr,gx,gy,data
if not valid_map(rmap) then rmap=-1
return,rmap & end

