;+
; Project     : SOHO-CDS
;
; Name        : FITS2MAP
;
; Purpose     : Make an image map from a FITS file
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : fits2map,file,map
;
; Examples    :
;
; Inputs      : FILE = FITS file name (or FITS data + HEADER)
;
; Opt. Inputs : None
;
; Outputs     : MAP = map structure
;
; Opt. Outputs: None
;
; Keywords    : SUB = [x1,x2,y1,y2] = indicies of subarray to extract
;               SOHO = designate as SOHO-viewed
;               OUTSIZE = image dimensions of map
;               HEADER = FITS header (output)
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 January 1998, D. Zarro, SAC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

pro fits2map,file,map,outsize=outsize,soho=soho,err=err,$
                  _extra=extra,header=fhead,sub=sub

on_error,1
err=''

;-- check inputs

if datatype(file) ne 'STR' then begin
 pr_syntax,'fits2map,file,map'
 return
endif

nf=n_elements(file)
fl=strarr(nf)
for i=0,nf-1 do fl(i)=loc_file(file(i),count=count,err=err)
ok=where(trim(fl) ne '',count)
if count eq 0 then begin
 message,err,/cont & return
endif
files=file(ok)

;-- try the new way

mreadfits,file,index,data,outsize=outsize,header=fhead,_extra=extra
index2map,index,data,map,soho=soho,sub=sub

return

;-- skip the old way

file=fl(0)
err=''
fxread,file,fdata,fhead,err=err
if err ne '' then begin
 message,err,/cont
 return
endif

get_fits_par,fhead,xc,yc,dx,dy,time=time,stc=stc,err=err
if err ne '' then return

if tag_exist(stc,'INSTRUME') then inst=trim(stc.instrume) else inst=''
if tag_exist(stc,'CROTA') then roll=float(stc.crota) else roll=0.
if tag_exist(stc,'DETECTOR') then det=trim(stc.detector)  else det=''
if tag_exist(stc,'exptime') then dur=float(stc.exptime) else dur=0.
if dur eq 0. then begin
 if tag_exist(stc,'date_obs') and tag_exist(stc,'date_end') then $
  dur=anytim2tai(stc.date_end)-anytim2tai(stc.date_obs)
endif
if tag_exist(stc,'telescop') then soho=(strpos(stc.telescop,'SOHO') gt -1) else $
 soho=keyword_set(soho)
if tag_exist(stc,'wavelnth') then wave=stc.wavelnth else wave=''
if tag_exist(stc,'origin') then origin=stc.origin else origin=''
id=trim(inst+' '+det+' '+wave+' '+origin)
if tag_exist(stc,'cunit1') then units1=trim(stc.cunit1)
if tag_exist(stc,'cunit2') then units2=trim(stc.cunit2)

if exist(units1) and exist(units2) then begin
 if units1 ne units2 then begin
  err='Spacing scale must be the same in both X- and Y- directions'
  message,err,/cont
  return
 endif
 units=units1
endif

;-- rebin image?

if exist(outsize) then begin
 msize=float([outsize(0),outsize(n_elements(outsize)-1)])
 sz=float(size(fdata))
 if (sz(1) ne msize(0)) or (sz(2) ne msize(1)) then begin
  fdata=congrid(temporary(fdata),msize(0),msize(1))
  dx=dx*sz(1)/msize(0)
  dy=dy*sz(2)/msize(1)
 endif
endif

;-- extract sub-array

if exist(sub) then begin
 if n_elements(sub) ne 4 then begin
  exptv,fdata
  wshow
  sub_data=tvsubimage(fdata,x1,x2,y1,y2)
  sub=[x1,x2,y1,y2]
 endif
endif

;-- make map
 
map=make_map(fdata,xc=xc,yc=yc,dx=dx,dy=dy,id=id,time=time,dur=dur,$
             sub=sub,roll=roll,soho=soho,units=units)

return

end
