;+
; Project     : SOHO-CDS
;
; Name        : GET_MAP_PROP
;
; Purpose     : extract map properties
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : prop=get_map_prop(map,/prop_name)
;
; Examples    :
;
; Inputs      : MAP = image map
;
; Opt. Inputs : None
;
; Outputs     : PROP = extracted property
;               (e.g. xc=get_map_prop(map,/xc) to extract map center
;
; Opt. Outputs: None
;
; Keywords    : ERR = error string
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 16 April 1998, D. Zarro, SAC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function get_map_prop,map,_extra=extra

on_error,1
err=''
error=1-valid_map(map,err=err,old=old_format)
if error or (datatype(extra) ne 'STC') or (n_params() ne 1) then begin
 pr_syntax,'prop=get_map_prop(map,[/xc,/yc,/id,/time,/xp,/yp])'
 return,-1
endif

tags=tag_names(extra)
if n_elements(tags) gt 1 then begin
 err='cannot handle more than one keyword'
 message,err,/cont
 return,-1
endif

;-- data properties

nx_pos=where(strpos(tags,'NX') gt -1,count)
if count gt 0 then return,data_chk(map(0).data,/nx)

ny_pos=where(strpos(tags,'NY') gt -1,count)
if count gt 0 then return,data_chk(map(0).data,/ny)

dr_pos=where(strpos(tags,'DR') gt -1,count)
if count then return,[min(map.data),max(map.data)]

xp_pos=where(strpos(tags,'XP') gt -1,count)
if count then return,get_map_xp(map)

yp_pos=where(strpos(tags,'YP') gt -1,count)
if count then return,get_map_yp(map)

xr_pos=where(strpos(tags,'XR') gt -1,count)
if count then return,get_map_xrange(map)

yr_pos=where(strpos(tags,'YR') gt -1,count)
if count then return,get_map_yrange(map)
       

;-- spacings

mspace=get_map_space(map)
space_pos=where(strpos(tags,'SPA') gt -1,count)
if count gt 0 then return,mspace

dx_pos=where(strpos(tags,'DX') gt -1,count)
if count then return,mspace(0)

dy_pos=where(strpos(tags,'DY') gt -1,count)
if count then return,mspace(1)

;-- center

mcenter=get_map_center(map)
cent_pos=where(strpos(tags,'CEN') gt -1,count)
if count gt 0 then return,mcenter

xc_pos=where(strpos(tags,'XC') gt -1,count1)
if count1 then return,mcenter(0)

yc_pos=where(strpos(tags,'YC') gt -1,count1)
if count1 then return,mcenter(1)


       
prop=tags(0)
present=grep(prop,tag_names(map),index=index)
if index gt -1 then return,map.(index) else message,prop+' property not supported',/cont
return,-1
end
