;+
; Project     : SOHO-CDS
;
; Name        : GRID_MAP
;
; Purpose     : Regrid an image map 
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : gmap=grid_map(map,gx,gy)
;
; Examples    :
;
; Inputs      : MAP = image map structure
;               GX,GY = new grid dimensions
;
; Opt. Inputs : None
;
; Outputs     : GMAP = regridded map
;
; Opt. Outputs: None
;
; Keywords    : SPACE = [sx,sy] = new grid spacing [def= auto determine]
;                or
;               DX=DX, DY=DY
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 August 1997, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function grid_map,map,gx,gy,dx=dx,dy=dy,space=space,err=err,_extra=extra

on_error,1
err=''

;-- check inputs (valid map & grid dimensions)

space_input=exist(space) or exist(dx) or exist(dy)
dim_input=exist(gx)

if (not valid_map(map,old=old)) or ((not space_input) and (not dim_input)) then begin
 pr_syntax,'gmap=grid_map(map,gx,gy,[space=space)'
 if exist(map) then return,map else return,-1
endif

if dim_input then if not exist(gy) then gy=gx

;-- determine for keywords or from map itself 

if space_input then begin

 if exist(dx) or exist(dy) then begin
  mspace=[1.,1.]
  if exist(dx) then mspace(0)=dx else begin
   xp=get_map_xp(map,dx=dx,err=err)
   if err ne '' then begin
    message,err,/cont
    return,map
   endif
   mspace(0)=dx
  endelse
  if exist(dy) then mspace(1)=dy else begin
   yp=get_map_yp(map,dy=dy,err=err)
   if err ne '' then begin
    message,err,/cont
    return,map
   endif
   mspace(1)=dy
  endelse

 endif else mspace=float([space(0),space(n_elements(space)-1)])
endif

;-- if user doesn't specify spacings and dimensions simultaneously then go 
;   the fast rebin route

if space_input and (not dim_input) then return,respace_map(map,mspace(0),mspace(1))
if dim_input and (not space_input) then return,rebin_map(map,gx,gy)

msize=float([gx,gy])
for i=0,n_elements(map)-1 do begin
 err=''
 tmap=map(i)
 if old then tmap=mk_map_new(temporary(tmap))
 unpack_map,tmap,data,xp,yp,dx=dx,dy=dy,nx=nx,ny=ny,err=err
 if err eq '' then begin
  if exist(mspace) then dspace=mspace else dspace=[dx,dy]
  if exist(msize) then dsize=msize else dsize=[nx,ny]
  if not old then begin
   if (dspace(0) eq dx) and (dspace(1) eq dy) and $
      (dsize(0) eq nx) and (dsize(1) eq ny) then begin
    message,'no gridding necessary',/cont
    goto,jump
   endif
  endif
  grid_xy,xp,yp,xg,yg,size=dsize,space=dspace
  data=interp2d(temporary(data),xp,yp,xg,yg,_extra=extra)
  if (nx ne dsize(0)) or (ny ne dsize(1)) then $
   tmap=rep_tag_value(tmap,data,'DATA') else $
    tmap.data=temporary(data)
  tmap.xc=get_img_center(xg,dx=dx)
  tmap.yc=get_img_center(yg,dy=dy)
  tmap.dx=dx
  tmap.dy=dy
jump:
  gmap=merge_struct(gmap,tmap)
 endif
endfor

return,gmap & end

