;+
; Project     : SOHO, YOHKOH
;
; Name        : INDEX2MAP
;
; Purpose     : Make an image map from index/data pair
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : index2map,index,data,maparr
;
; Examples    :
;
; Inputs      : index - vector of 'index' structures (per mreadfits/fitshead2struct)
;               data  - 2D or 3D
;
; Opt. Inputs : None
;
; Outputs     : maparr - 2D or 3D array of corresponding 'map structures'
;
; Opt. Outputs: None
;
; Keywords    : soho - if set, force SOHO-view (default uses index.TELESCOP if present)
;               sub  - if switch, interactively select subfield
;                      if 4 element vector, assume => [x1,x2,y1,y2] 
;               outsize - new output dimensions
;               positive - set negative points to zero
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written, 14 February 1998, S.L. Freeland (LSAL)
;               Embellished, 22 March 1998, D. Zarro (SAC/GFSC)
;
; Contact     : dzarro@solar.stanford.edu
;-

pro index2map, index, data, maparr, noclobber=noclobber, outsize=outsize,$
               soho=soho, sub=sub, ref_sub=ref_sub,positive=positive,$
               _extra=extra
on_error,1
err=''

usesub=n_elements(sub) gt 0 or n_elements(sub_ref) gt 0  ; sub FOV?
if n_elements(sub_ref) eq 0 then sub_ref=0               ; default->1st 
nimg=n_elements(index)

; ----------- time --------------

time=anytim(index,/vms)

;-------- coordinates ------------

soho=keyword_set(soho) or $
     strpos(string(gt_tagval(index,/telescop)),'SOHO') ne -1

dx=float(gt_tagval(index,/cdelt1,found=found1))
dy=float(gt_tagval(index,/cdelt2,found=found2))

if not (found1 and found2) then begin
 if tag_exist(index,'radius') then begin
  h=float(pb0r(time,/arc,soho=soho))
  dx=h(2)/index.radius
  dy=dx
 endif else begin
  err='cannot determine image scale'
  message,err,/cont
  return
 endelse
endif

xcen=float((gt_tagval(index,/naxis1))*dx)/2.
ycen=float((gt_tagval(index,/naxis2))*dy)/2.
xorg=float((gt_tagval(index,/crpix1))*dx)
yorg=float((gt_tagval(index,/crpix2))*dy)

xp=xcen-xorg
yp=ycen-yorg
cv1=gt_tagval(index,/crval1,found=found1)
cv2=gt_tagval(index,/crval2,found=found2)
cv3=gt_tagval(index,/crota,found=found3)

if found1 then xp=xp+cv1
if found2 then yp=yp+cv2
if found3 then roll=cv3 else roll=replicate(0.,nimg)

blank=replicate('',nimg)
if tag_exist(index,'INSTRUME') then inst=strtrim(index.instrume,2) else inst=blank
if tag_exist(index,'DETECTOR') then det=strtrim(index.detector,2)  else det=blank
if tag_exist(index,'exptime') then dur=float(index.exptime) else $
 dur=replicate(0.,nimg)
if total(dur) eq 0. then begin
 if tag_exist(index,'date_obs') and tag_exist(index,'date_end') then $
  dur=anytim2tai(index.date_end)-anytim2tai(index.date_obs)
endif

if tag_exist(index,'telescop') then soho=(strpos(index.telescop,'SOHO') gt -1) else soho=replicate(0,nimg)
if tag_exist(index,'wavelnth') then wave=strtrim(index.wavelnth,2) else wave=blank
if tag_exist(index,'origin') then origin=strtrim(index.origin,2) else origin=blank
id=trim(inst+' '+det+' '+wave)

;-- check if rebin requested

if exist(outsize) then begin
 msize=float([outsize(0),outsize(n_elements(outsize)-1)])
endif

delvarx,maparr
for i=0,nimg-1 do begin                                    ; 3D loop
 fdata=reform(data(*,*,i))
 sz=float(size(fdata))
 nx=sz(1) & ny=sz(2)

;-- only make maps of images with same dimensions as first image

 if (i eq 0) then begin
  nxr=nx & nyr=ny
  do_map=1
 endif else do_map=(nx eq nxr) and (ny eq nyr)

 if do_map then begin
  if exist(msize) then begin
   if (nx ne msize(0)) or (ny ne msize(1)) then begin
    fdata=congrid(temporary(fdata),msize(0),msize(1))
    dx(i)=dx(i)*nx/msize(0)
    dy(i)=dy(i)*ny/msize(1)
   endif
  endif

; ***** sub should be defined outside of this routine *******

  if usesub and n_elements(sub) ne 4 then begin   ; interactive
   exptv,fdata
   wshow
;   xloadct
   sub_data=tvsubimage(fdata,x1,x2,y1,y2)
   sub=[x1,x2,y1,y2]
  endif

  if keyword_set(positive) then fdata=temporary(fdata) > 0
  mapi=make_map(temporary(fdata),xc=xp(i),yc=yp(i),dx=dx(i),dy=dy(i),_extra=extra,$
         id=id(i),time=time(i),dur=dur(i),sub=sub,roll=roll(i),soho=soho(i))
  maparr=merge_struct(maparr,temporary(mapi))
 endif
endfor 

return

end
