;+
; Project     : SOHO-CDS
;
; Name        : MAKE_MAP
;
; Purpose     : Make an image map 
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : map=make_map(data)
;
; Examples    :
;
; Inputs      : DATA = 2d data array
;
; Opt. Inputs : None
;
;
; Outputs     : 
;               MAP ={data:data,xc:xc,yc:yc,dx:dx,dy:dy,time:time}
;               where,
;               DATA  = 2d image array
;
;               or old format (which is more memory intensive),
;
;               MAP ={data:data,xp:xp,yp:yp,time:time}
;               where,
;               XP,YP = 2d cartesian coordinate arrays (old format)
;
;               OLD_FORMAT is used if XC and YC
;               are 2d arrays of coordinates for the center of each pixel.
;
; Opt. Outputs: None
;
; Keywords    : XC,YC = center of image
;               DX,DY = pixel spacing in X and Y directions
;               TIME = image time
;               SUB   = [x1,x2,y1,y2] = indicies of sub-array to extract
;               DATA_UNITS = units of subarray in data units [def=pixel number]
;               OLD_FORMAT = use old .xp, .yp format
;               ROLL = image roll (deg clockwise from N)
;               RCENTER = roll center
;               FOV = same as SUB, but with /DATA_UNITS
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 October 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function make_map,data,xc=xcen,yc=ycen,dx=dx,dy=dy,time=time,$
                  sub=sub,roll=roll,old_format=old_format,$
                  data_units=data_units,err=err,rcenter=rcenter,$
                  _extra=extra,fov=fov

;-- check input image

err=''
sz=size(data)
if sz(0) ne 2 then begin
 err='Input image must be 2-d'
 message,err,/cont
 return,0
endif

if n_params() ne 1 then begin
 pr_syntax,'map=make_map(data,xc=xcen,yc=ycen,dx=dx,dy=dy)'
 return,0
endif

;-- check time

stime=anytim2utc(time,/vms,err=err)
if err ne '' then stime=anytim2utc(!stime,/vms)
err=''

;-- some defaults (origin center, unit pixel scale)

if not exist(xcen) then xcen=0.
if not exist(ycen) then ycen=0
if not exist(dx) then dx=1.
if not exist(dy) then dy=dx
nx=sz(1) & ny=sz(2)
dx=float(dx(0)) & dy=float(dy(0))
if dx eq 0. then dx=1.
if dy eq 0. then dy=1.

;-- extract subarray?

data_units=keyword_set(data_units)
old_format=keyword_set(old_format)
if n_elements(fov) eq 4 then begin
 subreg=fov & data_units=1
endif else begin
 if n_elements(sub) eq 4 then subreg=sub
endelse

sx=size(xcen) & sy=size(ycen)
old_input_format= (sx(0) eq 2) and (sy(0) eq 2) and $
                  (sx(1) eq nx) and (sy(1) eq nx) and $
                  (sx(2) eq ny) and (sy(2) eq ny)

if old_input_format then begin
 xp=xcen                
 yp=ycen
 xc=get_img_center(xp,dx=dx)
 yc=get_img_center(yp,dy=dy)
endif else begin
 xc=float(xcen(0)) & yc=float(ycen(0))
 xp=mk_map_xp(xc,dx,nx,ny)
 yp=mk_map_yp(yc,dy,nx,ny)
endelse

;-- start building object

prefix='map={data:data'

if n_elements(subreg) eq 4 then begin
 if data_units then begin
  sub=get_map_region(xp,yp,subreg,count=count,err=err)
  if (count gt 0) then begin
   if ( (sub(1)-sub(0)) lt 2) or ((sub(3)-sub(2)) lt 2) then begin
    err='Insufficient data points to produce map'
    return,0
   endif
   xp=temporary(xp(sub(0):sub(1),sub(2):sub(3)))
   yp=temporary(yp(sub(0):sub(1),sub(2):sub(3)))
  endif else begin
   err='Zero data points within selected subregion'
   message,err,/cont
   return,0
  endelse
  sub_data=data(sub(0):sub(1),sub(2):sub(3))
 endif else begin
  if min(subreg) lt 0 then begin
   err='Negative index values -> use /data for such values'
   return,0
  endif
  x1=subreg(0) < (nx-1) & x2=subreg(1) < (nx-1)
  y1=subreg(2) < (ny-1) & y2=subreg(3) < (ny-1)
  xp=temporary(xp(x1:x2,y1:y2))
  yp=temporary(yp(x1:x2,y1:y2))
  sub_data=data(x1:x2,y1:y2)
 endelse
 xc=get_img_center(xp)
 yc=get_img_center(yp)
 prefix='map={data:sub_data'
endif

;-- add properties

if old_format then begin
 prefix=prefix+',xp:xp,yp:yp'
endif else begin
 prefix=prefix+',xc:xc,yc:yc,dx:dx,dy:dy'
endelse

state=prefix+',time:stime'
if exist(roll) then state=state+',roll:float(roll)'
if (n_elements(rcenter) eq 2) and exist(roll) then $
 state=state+',roll_center:float(rcenter)'
state=state+'}'

s=execute(state)

if valid_map(map) and datatype(extra) eq 'STC' then map=join_struct(map,extra)

return,map

end


