;+
; Project     : SOHO-CDS
;
; Name        : MAP2FITS
;
; Purpose     : write image map to FITS file
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : map2fits,map,file
;
; Examples    :
;
; Inputs      : MAP = image map structure
;               FILE = FITS file name
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: None
;
; Keywords    : ERR = error string
;               BYTE_SCALE = byte scale data
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 August 1997, D. Zarro, SAC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

   pro map2fits,map,file,err=err,byte_scale=byte_scale

   err=''
   on_error,1

   if (not valid_map(map)) or (datatype(file) ne 'STR') then begin
    pr_syntax,'map2fits,map,file'
    return
   endif

   np=n_elements(map)
   if np ne n_elements(file) then begin
    err='Output file must be string array sinc map contains multiple images'
    message,err,/cont
    return
   endif

;-- form output file name

   cd,curr=cdir

   for i=0,np-1 do begin
    break_file,file(i),dsk,dir,dfile,dext
    if trim(dsk+dir) eq '' then fdir=cdir else fdir=dsk+dir
    if test_open(fdir,/write,err=err) then begin
;     if trim(dext) eq '' then fext='.fts' else fext=dext
     fext=dext
     filename=concat_dir(fdir,dfile)+fext

;-- unpack data

     unpack_map,map(i),data,xp,yp,dx=dx,dy=dy,xc=xc,yc=yc,err=err

;-- scale data?

     if keyword_set(byte_scale) then bscale,data,top=255

;-- add header for the output array.

     fxhmake,header,data,/date

;-- add FITS parameters CRPIX, CRVAL, etc.

     fxaddpar, header, 'ctype1', 'solar_x','Solar X (cartesian west) axis'
     fxaddpar, header, 'ctype2', 'solar_y','Solar Y (cartesian north) axis'

     fxaddpar, header, 'cunit1', 'arcsecs','Arcseconds from center of sun'
     fxaddpar, header, 'cunit2', 'arcsecs','Arcseconds from center of sun'

     fxaddpar, header, 'crpix1', 1, 'Reference pixel along X dimension'
     fxaddpar, header, 'crpix2', 1, 'Reference pixel along Y dimension'

     fxaddpar, header, 'crval1',xp(0,0)+dx/2., 'Reference position along X dimension'
     fxaddpar, header, 'crval2',yp(0,0)+dy/2., 'Reference position along Y dimension'

     fxaddpar, header, 'cdelt1',dx,'Increments along X dimension'
     fxaddpar, header, 'cdelt2',dy,'Increments along Y dimension'

     fxaddpar,header,'date_obs',map(i).time,'Observation date'
     if tag_exist(map,'dur') then fxaddpar,header,'exptime',map(i).dur,'Exposure duration'
     fxaddpar,header,'origin',map(i).id,'Data description'

     if tag_exist(map,'soho') then begin
      if map(i).soho then fxaddpar,header,'telescope','SOHO','Telescope'
     endif

;-- write out the file
   
     fxaddpar, header, 'filename', filename
     fxwrite, filename, header,data
    endif
   endfor

   return & end
