;+
; Project     : SOHO-CDS
;
; Name        : MK_CDS_IMAP
;
; Purpose     : Make an image map from a CDS QL structure
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : map=mk_cds_imap(ql)
;
; Examples    :
;
; Inputs      : QL = CDS quicklook data stucture
;
; Opt. Inputs : None
;
; Outputs     : MAP = map structure
;
; Opt. Outputs: None
;
; Keywords    : SUM   = sum intensities over window wavelength
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 October 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

  function mk_cds_imap,aa,window,sum=sum,_extra=extra,err=err

  err=''
  qlmgr,aa,valid
  if not valid then begin
   err='Invalid QL structure'
   return,-1
  endif
  dim=gt_dimension(aa)
  if dim.ssolar_x eq 1 then begin
   err='Singular X-dimension. Use MK_CDS_SMAP'
   message,err,/cont
   return,-1
  endif

  w=gt_wlimits(aa,/wave,/quiet)
  if not exist(window) then window=gt_cds_window(aa)
  if window(0) lt 0 then begin
   err='Aborted'
   return,-1
  endif

  nw=n_elements(window)
  dur=float(gt_duration(aa,/sec))
  time=gt_start(aa,/vms)
  for i=0,nw-1 do begin

   if (aa.detdesc(window(i)).ixstop(0) lt 0) then begin
    err='Zero data in window'
   endif else begin

    wlim=w(0:1,window(i))
    mlam=total(wlim)/n_elements(wlim)
    if keyword_set(sum) then begin
     h=float(gt_bimage(aa,wlim(0),wlim(1),xs=xp,ys=yp)) > 0.
    endif else begin
     h=float(gt_mimage(aa,mlam,xs=xp,ys=yp)) > 0.
    endelse

;-- make map

    xc=get_img_center(xp,dx=dx)
    yc=get_img_center(yp,dy=dy)

    tmap=make_map(temporary(h),xc=xc,yc=yc,dx=dx,dy=dy,units='arcsecs',_extra=extra,$
       time=time,dur=dur,units='arcsecs',$
       id=cds_wave_label((aa.detdesc.label)(window(i))),soho=1)
    map=merge_struct(map,tmap)
   endelse
  endfor
  return,map

  end
