;+
; Project     : SOHO-CDS
;
; Name        : MK_CDS_MAP
;
; Purpose     : Make an image map from a CDS QL structure
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : map=mk_cds_map(ql,window)
;
; Examples    :
;
; Inputs      : QL = CDS quicklook data stucture or CDS FITS file
;               WINDOW = window number
;
; Opt. Inputs : None
;
; Outputs     : MAP = map structure
;
; Opt. Outputs: None
;
; Keywords    : CLEAN = clean Cosmic ray hits
;               PEAK   = use peak intensity
;               UPDATE = update CDS pointing
;               DEBIAS = debias CDS data
;               CALIB = calibrate data
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 October 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function mk_cds_map,ql,window,clean=clean,sum=sum,update=update,$
 debias=debias,calib=calib,fdata=fdata,peak=peak,_extra=extra

if datatype(ql) eq 'STR' then begin
 fdata=readcdsfits(ql)
 if datatype(fdata) ne 'STC' then return,0
endif else begin
 if datatype(ql) eq 'STC' then fdata=ql else begin
  message,'enter a CDS QL structure',/cont &  return,0
 endelse
endelse

for i=0,n_elements(fdata)-1 do begin
 aa=fdata(i)
 if not exist(window) then window=gt_cds_window(aa)
 if window(0) lt 0 then return,-1

;-- update pointing

 if keyword_set(update) then begin
  err=''
  upd_cds_point,aa,err=err
  if err ne '' then message,err,/cont
 endif

;-- clean

 if keyword_set(clean) then cds_clean,aa

;-- calibrate

 if keyword_set(calib) then begin
  err=''
  vds_calib,aa,err=err
  if err ne '' then message,err,/cont
 endif

;-- produce image

 sum=keyword_set(sum) or (1-keyword_set(peak))
 dim=gt_dimension(aa)
 if dim.ssolar_x eq 1 then tmap=mk_cds_smap(aa,window,_extra=extra) else $
  tmap=mk_cds_imap(aa,window,sum=sum,_extra=extra)
 map=merge_struct(map,tmap)
endfor

return,map

end
