;+
; Project     : SOHO-CDS
;
; Name        : MK_EIT_MAP
;
; Purpose     : Make an image map from EIT FITS data
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : map=mk_eit_map(data,header) or mk_eit_map(index,data)
;
; Examples    :
;
; Inputs      : DATA,HEADER = FITS image/header combination
;               INDEX,DATA  = index/data combination
;
; Opt. Inputs : None
;
; Outputs     : MAP = map structure
;
; Opt. Outputs: None
;
; Keywords    : SUB = set to compute subarray
;               DEGRID = set to degrid
;               SIZE or OUTSIZE = output size of data
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 January 1997, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-


function mk_eit_map,index,data,degrid=degrid,sub=sub,size=gsize,$
            outsize=outsize,_extra=extra

on_error,1

data_head=(datatype(data) eq 'STR') and (size(index))(0) eq 2
index_data=(datatype(index) eq 'STC') and $
           (data_chk(index,/nx) eq (data_chk(data,/nim)))

if (1-data_head) and (1-index_data) then begin
 pr_syntax,'eit_map=mk_eit_map(data,header) or mk_eit_map(index,data)'
 return,-1
endif

;-- degrid?

if keyword_set(degrid) then begin
 message,'degridding disabled. Use EIT_PREP before calling MK_EIT_MAP',/cont
endif

;-- INDEX/DATA is the preferred new way

if not exist(gsize) and exist(outsize) then gsize=outsize
if index_data then begin
 index2map, index, data, map,sub=sub,_extra=extra,$
            outsize=gsize,units='arcsecs',/positive
 return,map
endif

;-- decode header (this is the old way)

get_fits_par,data,xc,yc,dx,dy,time=time,stc=stc,err=err
if err ne '' then return,-1

;-- rebin image?

image=index
if exist(gsize) then begin
 msize=float([gsize(0),gsize(n_elements(gsize)-1)])
 sz=float(size(index))
 if (sz(1) ne msize(0)) or (sz(2) ne msize(1)) then begin
  image=congrid(temporary(image),msize(0),msize(1))
  dx=dx*sz(1)/msize(0)
  dy=dy*sz(2)/msize(1)
 endif
endif

;-- extract sub-array

if keyword_set(sub) then begin
 exptv,alog10(index > .01)
 wshow
 sub_data=temporary(tvsubimage(image,x1,x2,y1,y2))
 delvarx,sub_data
 sub_array=[x1,x2,y1,y2]
endif

;-- make map
 
map=make_map(temporary(image) > 0,xc=xc,yc=yc,dx=dx,dy=dy,time=time,dur=float(stc.exptime),$
     id='EIT:'+stc.wavelnth,/soho,sub=sub_array,units='arcsecs',_extra=extra)

return,map & end

