;+
; Project     : SOHO-CDS
;
; Name        : MK_SXT_MAP
;
; Purpose     : Make an image map from an SXT index/data structure
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : map=mk_sxt_map(index,data)
;
; Examples    :
;
; Inputs      : INDEX,DATA = index/data combination
;
; Opt. Inputs : None
;
; Outputs     : MAP = map structure
;
; Opt. Outputs: None
;
; Keywords    : DIMENSIONS = [nx,ny] image dimensions to select
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 January 1997, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function mk_sxt_map,index,data,use_hist=use_hist,$
         tstart=tstart,tstop=tstop,_extra=extra,dimensions=dimensions

if datatype(index) ne 'STC' then begin
 message,'enter an SXT index structure',/cont
 return,0
endif

;--  scaling info

xc=gt_center(index,/ang,/x,use_hist=use_hist)
yc=gt_center(index,/ang,/y,use_hist=use_hist)
dx=gt_pix_size(index)
dy=dx

;-- dimensions to filter

nd=n_elements(dimensions)
if nd gt 0 then begin
 sx=dimensions(0)
 sy=dimensions( 1 < (nd-1) )
endif

np=n_elements(index)
times=gt_day(index,/str)+' '+gt_time(index,/str)
if not exist(tstart) then tstart=anytim2tai(times(0)) else tstart=anytim2tai(tstart)
if not exist(tstop) then tstop=anytim2tai(times(np-1)) else tstop=anytim2tai(tstop)

for i=0,n_elements(index)-1 do begin
 err=''
 time=anytim2tai(times(i))
 if (time ge tstart) and (time le tstop) then begin

  nx=gt_shape(index(i),/x)
  ny=gt_shape(index(i),/y)
  if not exist(sx) then sx=nx
  if not exist(sy) then sy=ny

  if (nx eq sx) and (ny eq sy) then begin
   map=make_map(data(0:nx-1,0:ny-1,i) > 0,xc=xc,yc=yc,dx=dx,dy=dy,time=times(i),_extra=extra,$
        dur=gt_expdur(index(i))/1000.,id=gt_filtb(index(i),/str),$
        err=err,units='arcsecs')
  endif else err='Skipping image dimensions: '+trim(string(nx))+','+trim(string(ny))
  if err ne '' then begin
   message,err,/cont
  endif else begin
   if not exist(img) then img=temporary(map) else $
    img=merge_struct(temporary(img),temporary(map))
  endelse
 endif

endfor

if not exist(img) then begin
 message,'No images during specified times',/cont
 img=0
endif

return,img

end
