;+
; Project     : SOHO-CDS
;
; Name        : MOVIE_MAP
;
; Purpose     : make movie of series of map images
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : movie_map,map
;
; Examples    :
;
; Inputs      : MAP = array of map structures created by MAKE_MAP
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: None
;
; Keywords    : ROTATE = differentially rotate images
;               TREF = reference time to rotate to [def = first image]
;               GIF = make series of GIF files
;               MAX = data max to scale to
;               CMAP = contour map to overlay
;               CMIN,CMAX = min/max values for contoured data
;               TSTART/TSTOP = movie start/stop times
;               CROTATE = diff_rot contour images
;               GNAME = name for GIF frame [def = frame, e.g. frame00.gif]
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 November 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

pro movie_map,map,tref=tref,xsize=xsize,ysize=ysize,_extra=extra,trans=trans,$
 rotate=rotate,gif=gif,dmax=dmax,dmin=dmin,cmap=cmap,cmax=cmax,cmin=cmin,$
 tstart=tstart,tstop=tstop,use_cont=use_cont,ctol=ctol,crotate=crotate,$
 positive=positive,negative=negative,frames=frames,log=log,$
 tag_id=tag_id,remap=remap,gname=gname,drange=drange,velocity=velocity

if not valid_map(map) then begin
 pr_syntax,'movie_map,map'
 return
endif

;-- setup window

if datatype(gname) ne 'STR' then gname='frame'
get_xwin,index,xsize=xsize,ysize=ysize
window=!d.window
sx=!d.x_size & sy=!d.y_size

cont=exist(cmap)
if cont then begin
 ctimes=anytim2tai(cmap.time)
endif

;-- handle times

mtimes=anytim2tai(map.time)
mvtimes=mtimes
use_cont=keyword_set(use_cont)
if cont and use_cont then mvtimes=ctimes
tmin=min(mvtimes)
tmax=max(mvtimes)
err=''
pstart=anytim2tai(tstart,err=err)
if err ne '' then pstart=tmin
err=''
pstop=anytim2tai(tstop,err=err)
if err ne '' then pstop=tmax

ntimes=n_elements(mtimes)
do_movie=where( (mtimes ge pstart) and (mtimes le pstop),fcount)
if fcount eq 0 then begin
 message,'no images during specified TSTART/STOP',/cont
 return
endif

;-- handle frame numbers

frame=exist(frames)
if frame then begin
 do_movie=indgen(ntimes)
 if (n_elements(frames) gt 1) then begin
  do_movie=where_vector(frames,indgen(ntimes),fcount)
  if count eq 0 then begin
   message,'no images for specified frame numbers',/cont
   return
  endif
 endif
endif

;-- which tag to display?

if not exist(tag_id) then tag_id='DATA'
if keyword_set(velocity) then tag_id='centroid'
tag_no=get_tag_index(map,tag_id,err=err)
if tag_no lt 0 then begin
 message,err,/cont
 return
endif

;-- check if 3-d and get min/max for scaling

sz=size(map(0).data)
three_d=sz(0) eq 3

if n_elements(drange) eq 2 then begin
 if not exist(dmin) then dmin=drange(0)
 if not exist(dmax) then dmax=drange(1)
endif

if not exist(dmin) then begin
 if three_d then dmin=min(map(*,*,do_movie).(tag_no)) else $
  dmin=min(map(do_movie).(tag_no)) 
endif

if not exist(dmax) then begin
 if three_d then dmax=max(map(*,*,do_movie).(tag_no)) else $
  dmax=max(map(do_movie).(tag_no)) 
endif


if keyword_set(positive) then dmin=0.
if keyword_set(negative) then dmax=0.

dprint,'%dmax, ',dmax
dprint,'%dmin, ',dmin

;-- solar rotate?

rotate=keyword_set(rotate)
crotate=keyword_set(crotate)
err=''
time=anytim2tai(tref,err=err)
if (err ne '') and (rotate or crotate) then time=map(do_movie(0)).time

;-- make animate window

xkill,/all
if not frame then xinteranimate,set=[sx,sy,fcount]
cleanplot

if not exist(ctol) then ctol=0
for j=0,fcount-1 do begin
 i=do_movie(j)
 tmap=map(i)
; if three_d then tmap=reg_map(map,i) else tmap=map(i)
 if rotate then tmap=drot_map(tmap,time=time,noremap=1-keyword_set(remap)) 
 plot_map,tmap,xsize=xsize,ysize=ysize,window=window,_extra=extra,$
  dmin=dmin,dmax=dmax,trans=trans,positive=positive,negative=negative,$
  tag_id=tag_no,velocity=velocity,log=log
 if cont then begin
  diff=abs(anytim2tai(tmap.time)-ctimes)
  use=where(diff eq min(diff))
  dprint,'% base image time:    '+tmap.time
  dprint,'% contour image time: '+cmap(use).time
  if ctol eq 0 then do_it=1 else do_it=(min(diff) le ctol)
  if do_it then begin
   plot_map,cmap(use),_extra=extra,/over,max=cmax,min=cmin,rotate=crotate,$
    tref=time,/pos,lcolor=2,border=0
   plot_map,cmap(use),_extra=extra,/over,max=cmax,min=cmin,rotate=crotate,$
    tref=time,/neg,lcolor=5,border=0
  endif else dprint,'% diff: ',min(diff)
 endif
 window=!d.window

 if frame then gif=0
 if not frame then xinteranimate,frame=j,window=[window,0,0,sx,sy]
 if keyword_set(gif) then begin
  counter=trim(string(j))
  if j lt 10 then counter='0'+counter
  gif_file=gname+counter+'.gif'
  x2gif,gif_file
 endif 
endfor

if not frame then xinteranimate,/keep

if keyword_set(gif) then begin
 spawn,'ls '+gname+'*.gif > agif.lis'
 message,'created GIF filenames are listed in file: agif.lis',/cont
endif

return & end

