;+
; Project     : SOHO_CDS
;
; Name        : PLOT_MAP
;
; Purpose     : Plot an image map 
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : plot_map,map
;
; Examples    :
;
; Inputs      : MAP = image structure map created by MAKE_MAP
;
; Opt. Inputs : None
;
;
; Outputs     : None
;
; Opt. Outputs: None
;
; Keywords    : 
;     /OVER = overlay on previous image
;     /CONT = contour the image
;     /SMOOTH = smooth the image
;     FOV = [fx,fy] = field of view to be plotted 
;     GRID = grid spacing (deg) for latitude-longitude grid [def= 0, no grid]
;     GLABEL = label grid with coordinate values [def = 0, no labels]
;     COLOR = colorize contours
;     CENTER = [xc,yc] = center coordinates (arcsec) of FOV [def = center of image]
;            (if center is a valid map, then use its center)
;     DMIN,DMAX = min, max data for plot [def = data min,max]
;     BORDER = draw border around image [def = no]
;     DRAW = alerts PLOT_MAP that window is a DRAW widget
;     /TAIL = allows user to tailor contours
;     /LOG  = log_10 scale image
;     WINDOW = window index to send plot to
;     /NOAXES = inhibit plotting axes
;     /NOTITLE = inhibit printing title
;     /NOLABELS = inhibit axis labels
;     /VELOCITY = scale data for velocity color table
;     /ROTATE  = solar rotate image contour
;     LEVELS  = user specified contour levels
;     /POSITIVE = plot positive data
;     /NEGATIVE = plot negative data
;     XRANGE,YRANGE = cartesian plot limits 
;     /INTERLACE = interlace two images when overlaying
;     /COMPOSITE = simultaneously plot two images when overlaying
;                = type of compositing:
;                  1: original , 2: latest(new), 3: max(new/old) 4: min(new/old)
;     /AVERAGE   = average two images when using /COMPOSITE
;     BOTTOM = lowermost color index to use in color scaling [def=0]
;     LAST_SCALE = use MIN/MAX from previous plot, or if LAST is a valid map
;                  use scaling from it
;     ZOOM = switch on zoom mode
;     IDENT = plot map with specified ID
;     LIMB = overplot solar limb
;     REGION = data coordinates of zoomed region [xmin,xmax,ymin,ymax]
;     ROLL = roll image contour
;     BTHICK = border thickness
;
; Restrictions: None
;      - do not set /GRID unless absolute coordinates of image are known
;      - do not set /OVER unless a plot exists on the current device
;      - do not set /COLOR unless a color output device exists
;
; Side effects: None
;
; History     : Written 22 December 1996, D. Zarro, SAC/GSFC
;             : S.L.Freeland - let COMPOSITE have different interpretations
;                              
;
; Contact     : dzarro@solar.stanford.edu
;-

pro plot_map,map,cont=cont,over=over,smooth=smooth,border=border,$
 fov=fov,grid=grid,glabel=glabel,color=color,center=center,$
 draw=draw,tail=tail,log=log,notitle=notitle,$
 lcolor=lcolor,window=window,noaxes=noaxes,nolabels=nolabels,$
 xsize=xsize,ysize=ysize,new=new,rotate=rotate,levels=levels,$
 missing=missing,dmin=dmin,dmax=dmax,$
 top=top,velocity=velocity,quiet=quiet,$
 tag_id=tag_id,trans=trans,positive=positive,negative=negative,$
 centroid=centroid,offset=offset,time=time,bottom=bottom,charsize=charsize,$
 cstyle=cstyle,cthick=cthick,date_only=date_only,time_only=time_only,$
 font=font,arcsecs=arcsecs,last_scale=last_scale,composite=composite,$
 interlace=interlace,xrange=xrange,yrange=yrange,_extra=extra,$
 average=average,ncolors=ncolors,drange=drange,zoom=zoom,$
 ident=ident,err=err,limb=limb,roll=roll,rcenter=rcenter,$
 duration=duration,position=position,bthick=bthick,no16bit=no16bit

;-- sone variables saved in memory for overlay

common plot_map2,last_window,last_time,last_drange,last_top,last_bottom,$
       last_view,last_pos

on_error,1
err=''

;-- check input image

if not valid_map(map) then begin
 err='Invalid input image map'
 message,err,/cont
 return
endif

;-- check ID

if datatype(ident) eq 'STR' then begin
 found=strpos(trim(strupcase(map.id)),trim(strupcase(ident))) gt -1
 if not found then begin
  err='Specified ID not found'
  message,err,/cont
  return
 endif
endif

if n_elements(map) gt 1 then begin
 message,'Cannot handle more than one image map at this time',/cont
 return
endif

;-- examine keywords

quiet=keyword_set(quiet)
loud=1-quiet
have_time=tag_exist(map,'TIME')
have_id=tag_exist(map,'ID')
if not exist(tag_id) then tag_id='DATA'
if not exist(grid) then grid=0                   ;-- LAT-LON spacing

over=keyword_set(over)         ;-- overlay image
take_average=keyword_set(average)
rotate=keyword_set(rotate)     ;-- rotate overlay
cont=keyword_set(cont)         ;-- contour image
smo=keyword_set(smooth)        ;-- smooth image
bord=exist(border)             ;-- plot image border
log=keyword_set(log)           ;-- log scale image
droll=exist(roll)               ;-- roll image
arcsecs=keyword_set(arcsecs)

if arcsecs then afac=1. else afac=60.
if keyword_set(limb) and (grid eq 0) then grid=-1

;-- always overlay as a contour unless /interlace or /composite are set

interlace=keyword_set(interlace)
if keyword_set(composite) then comptype=composite   ; intercept COMPTYPE
composite=keyword_set(composite)
if composite or interlace then over=1
if over and (not composite) and (not interlace) then cont=1 

;-- colors

zbuff=!d.name eq 'Z'
if not exist(color) then color=0                 ;-- default no color
n_colors=!d.table_size 
white=n_colors-1L & black=0L
if not exist(lcolor) then begin
 if (!d.name eq 'X') or zbuff then lcolor=white else lcolor=black
endif
if not exist(bottom) then bottom=0 else bottom=bottom > 0
if not exist(top) then begin
 if exist(ncolors) then top=(bottom+ncolors-1L) else top=n_colors-1L
endif
top=top < (n_colors-1L)

;-- plot controls
 
if not exist(charthick) then charthick=1
if not exist(charsize) then charsize=1
if not exist(font) then font=-1
noaxes=keyword_set(noaxes)
words=data_chk(map.data,/type) ne 1         ; boolean for tv/tvrd
if keyword_set(no16bit) then words=0

;-- SOHO view?

if tag_exist(map,'SOHO') then soho_view=map.soho else soho_view=0
if soho_view then dprint,'% using SOHO view'

;-- solar rotate or roll image

if rotate or droll then begin
 if (not cont) then begin
  err='Can only solar rotate/roll in overlay or contour mode'
  message,err,/cont
  return
 endif
; time=-1
 if rotate then begin
  if exist(time) then rtime=time else if exist(last_time) and over then rtime=last_time
 endif else rtime=map.time
 rmap=drot_map(map,duration,time=rtime,/noremap,roll=roll,tag_id=tag_id,err=err,$
               center=rcenter,/verb)
 if err ne '' then return
endif else rmap=map

;-- unpack data and pixel arrays

unpack_map,rmap,pic,xarr,yarr,dx=dx,dy=dy,tag_id=tag_id,nx=nx,ny=ny,$
      velocity=velocity,centroid=centroid

;-- translate coordinates?

xoff=0. & yoff=0.
if (n_elements(trans) eq 2) then begin
 xoff=trans(0) & yoff=trans(1)
endif
if (xoff ne 0.) or (yoff ne 0.) then begin
 xarr=temporary(xarr)+xoff & yarr=temporary(yarr)+yoff
endif

;-- deduce image center 
;   (start with that of image, then FOV keyword, then CENTER keyword)

dcenter=get_map_center(rmap)
if valid_map(fov) then dcenter=get_map_center(fov)
if exist(center) then begin
 if valid_map(center) then dcenter=get_map_center(center) else begin
  if n_elements(center) eq 2 then dcenter=float(center)
 endelse
endif

;-- deduce FOV size
;   (start with that of image, then FOV keyword)

dfov=get_map_fov(rmap,arcmin=1-arcsecs,/round)
if exist(fov) then begin
 if valid_map(fov) then dfov=get_map_fov(fov,arcmin=1-arcsecs,/round) else begin
  nfov=n_elements(fov)
  dfov=float([fov(0),fov(nfov-1)])
 endelse
endif
half_fov= (afac*dfov)/2.     

;-- deduce actual ranges 
;   (start with image, then FOV, then XRANGE/YRANGE keywords)

dxrange=get_map_prop(rmap,/xr)
dyrange=get_map_prop(rmap,/yr)
dxrange=[dcenter(0)-half_fov(0),dcenter(0)+half_fov(0)]
dyrange=[dcenter(1)-half_fov(1),dcenter(1)+half_fov(1)]
if n_elements(xrange) eq 2 then dxrange=xrange
if n_elements(yrange) eq 2 then dyrange=yrange

;-- actual data limits

min_x=min(xarr) & max_x=max(xarr)
min_y=min(yarr) & max_y=max(yarr)
xb = [min_x,max_x,max_x,min_x,min_x]
yb = [min_y,min_y,max_y,max_y,min_y]

;-- viewport

xmin=min(dxrange) & xmax=max(dxrange)
ymin=min(dyrange) & ymax=max(dyrange)
if (xmin eq xmax) or (ymin eq ymax) then begin
 err='Plot scale MIN/MAX must differ'
 message,err,/cont
 return
endif
dcenter=[(xmin+xmax),(ymin+ymax)]/2.

dprint,'% xc,yc,dx,dy',dcenter(0),dcenter(1),dx,dy

;-- data value offsets

if exist(offset) then pic=temporary(pic)+offset
if keyword_set(positive) then pic=temporary(pic) > 0
if keyword_set(negative) then pic=temporary(pic) < 0

;-- plot labels

mtitle=''
if (not over) then begin
 if have_id then mtitle=rmap.id
 if have_time then begin
  err=''
  date_obs=anytim2utc(rmap.time,/vms,err=err,time_only=time_only,date_only=date_only)
  if err eq '' then mtitle=mtitle+' '+date_obs
 endif
endif
mtitle=trim(mtitle)

if tag_exist(map,'units') then units='('+map.units+')' else units='(arcsecs)'
xtitle='X '+units & ytitle='Y '+units
if keyword_set(nolabels) then begin
 xtitle='' & ytitle=''
endif
if keyword_set(notitle) then mtitle=''

;-- open a new window if one doesn't exist

if (not over) then begin

 if keyword_set(new) then delvarx,last_window
 if (not exist(window)) and exist(last_window) then window=last_window
 get_xwin,window,xsize=xsize,ysize=ysize,draw=draw,/noshow
 if not exist(xsize) then xsize=!d.x_size
 if not exist(ysize) then ysize=!d.y_size

;-- adjust position to ensure a square image 
;   (data units per device pixel is same in x- and y- directions)
 
 def_position=[.1,.1,.9,.9]
 if n_elements(position) eq 4 then dpos=position else dpos=def_position
 dev_pos=dpos
 dev_pos([0,2])=!d.x_vsize*dpos([0,2])
 dev_pos([1,3])=!d.y_vsize*dpos([1,3])
 skew=(ymax-ymin)/(xmax-xmin)
 new_dev_pos=(dev_pos(1)+(dev_pos(2)-dev_pos(0))*skew)
 if new_dev_pos gt ysize then begin
  ymax=ymin+(xmax-xmin)*(ysize-dev_pos(1))/(dev_pos(2)-dev_pos(0))
  ymin=2.*dcenter(1)-ymax
  dev_pos(3)=ysize
  print,(ymax-ymin)/(xmax-xmin)
  dyrange(1)=ymax
  dyrange(0)=ymin
 endif else dev_pos(3)=new_dev_pos
; < ysize
 dpos([0,2])=dev_pos([0,2])/!d.x_vsize
 dpos([1,3])=dev_pos([1,3])/!d.y_vsize

 dfac=(xmax-xmin)/(ymax-ymin)
 sfac=float(!d.x_ch_size)/float(!d.y_ch_size)
 dprint,'% sfac, dfac',sfac,dfac
 skew=sfac*dfac
 xmargin=!x.margin & ymargin=!y.margin
 cl=where_vector(xmargin,ymargin*skew,count)
 if (count eq 0) or exist(margin) then begin
  if not exist(margin) then margin=xmargin(0)
;  !x.margin=[margin,margin] & !y.margin=!x.margin*skew
;   xmargin=[margin,margin] & ymargin=!x.margin*skew
 endif 

;-- else get viewport from previous plot

endif else begin
 if exist(window) and exist(last_window) then begin
  if window ne last_window then begin
   err='WARNING - overlaying on other than last plot window. Cannot guarantee coalignment'
   print,'---------------------------------------------------------------------'
   message,err,/cont
   print,'---------------------------------------------------------------------'
  endif
 endif
 smart_window,last_window,status=status,draw=draw
 if not status then begin
  if exist(last_window) then app=trim(string(last_window))
  err='Could not open window '+app
  message,err,/cont
  return
 endif
 dpos=last_pos
 dxrange=!x.crange & dyrange=!y.crange
endelse

;-- determine FOV of sub-region

region=[xmin,xmax,ymin,ymax]

sub=get_map_region(xarr,yarr,region,count=count)
if (count le 2) then begin
 err='Only 2 data points within FOV' 
 if loud then message,err,/cont
 if not over then erase
 goto,done
endif

;-- extract FOV sub-region for image plots 

if (abs(sub(1)-sub(0)) lt 1) or (abs(sub(3)-sub(2)) lt 1) then begin
 err='Insufficient data within FOV' 
 if loud then message,err,/cont
 if not over then erase
 goto,done
endif

prange=[min(pic),max(pic)]
if count eq nx*ny then begin
 if loud then message,'fov > image size',/cont 
endif else begin
 tpic=temporary(pic(sub(0):sub(1),sub(2):sub(3)))
 pmin=min(tpic,max=pmax)
 prange=[pmin,pmax]
 if not cont then begin
  pic=copy_var(tpic)
  xarr=temporary(xarr(sub(0):sub(1),sub(2):sub(3)))
  yarr=temporary(yarr(sub(0):sub(1),sub(2):sub(3)))
  if loud then message,'fov < image size',/cont
  min_x=min(xarr) & max_x=max(xarr)
  min_y=min(yarr) & max_y=max(yarr)
  xb = [min_x,max_x,max_x,min_x,min_x]
  yb = [min_y,min_y,max_y,max_y,min_y]
  sz=size(pic) & nx=sz(1) & ny=sz(2)
  print,'-- image size reduced to: ',nx,ny
 endif
endelse

;-- sort out data limits
;   (start with image limits, then DMIN/DMAX, then LAST_SCALE, and then DRANGE)

if exist(dmin) then prange(0)=dmin
if exist(dmax) then prange(1)=dmax
if exist(last_scale) then begin
 if valid_map(last_scale) then prange=get_map_prop(last_scale,/dr) else begin
  if n_elements(last_drange) eq 2 then prange=last_drange
 endelse
endif
if exist(prange) then begin 
 if valid_map(prange) then prange=get_map_prop(prange,/dr) else begin
  if n_elements(drange) eq 2 then prange=drange
 endelse
endif

pcheck=where( (pic lt prange(0)) or (pic gt prange(1)),pcount)
if pcount eq n_elements(pic) then begin
 err='Insufficient data points to plot'
 if loud then message,err,/cont
 if not over then erase
 goto,done
endif

if pcount gt 0 then begin
 if exist(missing) then pic(pcheck)=missing
endif

;-- log image?

if log then begin
 pic=alog10(temporary(pic) > .01)
 prange=alog10(prange > .01)
endif

if prange(0) eq prange(1) then begin
 err='MIN data = MAX data'
 if loud then message,err,/cont
 if not over then erase
 goto,done
endif

;-- contour plot? (this mess needs cleaning up)

if cont then begin
 if n_elements(nlev) eq 0 then nlev=10
 if (n_elements(clev) eq 0) or over then $
  clev=prange(0)+findgen(nlev)*(prange(1)-prange(0))/(nlev-1.)
 if n_elements(clabel) eq 0 then clabel=intarr(nlev) 
 if n_elements(cthick) eq 0 then cthick=replicate(1.,nlev)
 if n_elements(cstyle) eq 0 then cstyle=replicate(0,nlev)
 colors=replicate(fix(lcolor),nlev)  
 in_levels=0
 if n_elements(levels) gt 1 then begin
  clev=levels
  in_levels=1
 endif
 if keyword_set(tail) then begin                ;-- tailor contours?
  if not in_levels then begin
   print,'* default is equispaced contours between:'
   print,' MIN = ',prange(0),' and MAX = ',prange(1)
   print,'* enter explicit values for contour levels'
   clev='' & read,'----> ',clev
   if clev eq '' then begin
    nlev='' & read,'* enter number of equispaced contour levels for image [def=10]: ',nlev
    if nlev eq '' then nlev=10 else nlev=fix(nlev)
    clev=prange(0)+findgen(nlev)*(prange(1)-prange(0))/(nlev-1.)
   endif else begin
    clev=float(str2arr(clev)) & cs=sort(clev) & clev=clev(cs)
   endelse
  endif
  if n_elements(clev) lt 2 then clev=[clev,max_pic]
  nlev=n_elements(clev) & clabel=intarr(nlev) 
  ans='' & read,'* label contours [def=n]? ',ans
  if ans eq '' then ans='n' & ans=strupcase(strmid(ans,0,1))
  if ans eq 'Y' then begin
   clabel=clev & odd=where(findgen(nlev) mod 2) & clabel(odd)=0.
  endif
  cstyle='' & read,'* enter contour linestyle [def=0, solid]: ',cstyle
  cstyle=replicate(fix(cstyle),nlev)
  cthick='' & read,'* enter contour thickness [def=1]: ',cthick
  if cthick eq '' then cthick=1. & cthick=replicate(float(cthick),nlev)
 endif
endif

;-- set up plotting scale (without plotting data yet)

if (not over) then begin
 plot,[xmin,xmax,xmax,xmin,xmin],[ymin,ymin,ymax,ymax,ymin],/data, $
  xstyle=5,ystyle=5,noeras=0,/nodata,xrange=dxrange,yrange=dyrange,$
  position=dpos
  smart_window,window,/set,draw=draw
endif 
 
dprint,'*******************'
dprint,'% sub ',sub
dprint,!x.margin
dprint,!y.margin
dprint,!x.crange
dprint,!y.crange
dprint,dxrange
dprint,dyrange
dprint,dpos
dprint,'*******************'

if cont then begin   
 cs=uniq([clev],sort([clev]))
 contour,pic,xarr,yarr,/data,xstyle=5,ystyle=5,c_linestyle=cstyle(cs),$
  levels=clev(cs),c_labels=clabel(cs),c_thick=cthick(cs),c_colors=colors(cs),$
  max_value=prange(1),min_value=prange(0),charsize=charsize,font=font,$
  spline=smo,/noeras,noclip=0,/follow,xrange=dxrange,yrange=dyrange,$
  clip=[dxrange(0),dyrange(0),dxrange(1),dyrange(1)],position=dpos

;-- image?

endif else begin

 xb_dev=!d.x_vsize*(!x.s(0)+!x.s(1)*xb)
 yb_dev=!d.y_vsize*(!y.s(0)+!y.s(1)*yb)
 sx=abs(max(xb_dev)-min(xb_dev)) > 1.
 sy=abs(max(yb_dev)-min(yb_dev)) > 1.

;-- rebin image for X-windows or Z-buffer (!d.name = 'X' or 'Z')
;-- or plot in Postscript using scalable pixels (!d.name = 'PS')

 mpic=prange
 if log then mpic=10.^prange
 dprint,'% min, max pic: ',mpic
 dprint,'% bottom, top: ',bottom,top
 pic=cscale(pic,top=top,bottom=bottom,max=prange(1),min=prange(0),/no_copy)

;-- check if composite/interlace is requested

 dprint,'% over, sx, sy:',over, sx,sy
 
 if (!d.name eq 'X') or zbuff then begin
  pic=congrid(temporary(pic),sx,sy,interp=smo)
  if (composite or interlace) and over then begin
   ok=(xb_dev(0) gt 1) and (xb_dev(0) lt xsize) and $
      (yb_dev(0) gt 1) and (yb_dev(0) lt ysize) and $
      (xb_dev(0)+sx lt xsize) and $
      (yb_dev(0)+sy lt ysize)

;-- just create a blank bytarr if underlying image cannot be read

   if ok then begin
    if zbuff then $
     base=tvrd(xb_dev(0),yb_dev(0),sx,sy,channel=words,words=words) else $
      base=tvrd(xb_dev(0),yb_dev(0),sx,sy)
   endif else base=bytarr(sx,sy)

   base=cscale(base,top=last_top,bottom=last_bottom,/no_copy)

;-- combine underlying and overlaying images 

   if interlace then begin
    base=swiss_cheese(base,last_bottom,/shift,/no_copy)
    pic=swiss_cheese(pic,bottom,/no_copy)
    pic=temporary(base)+temporary(pic)
   endif else begin                              ; COMPOSITE set
    case comptype of                            ; setable option
       1: begin                                 ; backwardly compatible
	   bfact=([1,2])(take_average)
           pic=temporary(base)/bfac+temporary(pic)/bfac
          end
       3: pic=temporary(base) > temporary(pic)        ; 'largest'  pixel
       4: pic=temporary(base) < temporary(pic)        ; 'smallest' pixel
       else: do_nothing=1
    endcase
   endelse

  endif
 endif

 dprint,'% words: ',words

 case 1 of
  zbuff: tv,pic,xb_dev(0),yb_dev(0),xsize=sx,ysize=sy, channel=words,words=words
  else:  tv,pic,xb_dev(0),yb_dev(0),xsize=sx,ysize=sy
 endcase

endelse

;-- plot axes

done:

if (not over) then begin
 if noaxes then begin
  xstyle=5 & ystyle=5
 endif else begin
  xstyle=1 & ystyle=1
 endelse
 plot,[xmin,xmax,xmax,xmin,xmin],[ymin,ymin,ymax,ymax,ymin],/data,noclip=0, $
  xstyle=xstyle,ystyle=ystyle,/noeras,/nodata,xrange=dxrange,$
  yrange=dyrange,color=lcolor,xtitle=xtitle,ytitle=ytitle,title=mtitle,$
  _extra=extra,charsize=charsize,font=font,position=dpos
  clip=[dxrange(0),dyrange(0),dxrange(1),dyrange(1)]
endif

;-- overlay a solar latitude-longitude grid

if (grid ne 0) and (have_time) then begin
 if (not over) or composite or interlace then begin
  plot_helio,rmap.time,roll=roll,grid=grid,glabel=glabel,$
   /noeras,color=lcolor,font=font,soho=soho_view
 endif
endif

;-- plot border edges

if bord then begin
 if not exist(bthick) then bthick=1
 if border eq 2 then begin
  edge=replicate(1.,nx,ny) 
  if ((nx gt 2) and (ny gt 2)) then edge(1:nx-2,1:ny-2) = 0
  contour,edge,xarr,yarr,xstyle=5,ystyle=5,levels=[0,1],$
   /data,noclip=0,/noeras,/follow,spline=smo,c_linestyle=[0,0],c_thick=bthick,$
   xrange=dxrange,yrange=dyrange,c_color=[lcolor,lcolor],position=dpos
 endif else begin
  dx2=dx/2. & dy2=dy/2.
  xedge=[xmin-dx2,xmax+dx2,xmax+dx2,xmin-dx2,xmin-dx2]
  yedge=[ymin-dy2,ymin-dy2,ymax+dy2,ymax+dy2,ymin-dy2]
  oplot,xedge,yedge,thick=bthick
 endelse
endif

;-- zooming

if keyword_set(zoom) then begin
 sregion=drawbox(/data)
 zregion=[sregion(0),sregion(2),sregion(1),sregion(3)]
 plot_map,map,_extra=extra,$
  grid=grid,last=map,log=log,xrange=[zregion(0),zregion(1)],$
                             yrange=[zregion(2),zregion(3)]

endif

;-- save last settings

if not over then begin
 if exist(window) then last_window=window
 if exist(prange) then begin
  last_drange=prange
  if log then last_drange=10.^last_drange
 endif
 last_time=map.time
 if tag_exist(map,'rtime') then last_time=map.rtime
 last_top=top
 last_bottom=bottom
 last_view=soho_view
 last_pos=dpos
endif

;-- restore margins

if exist(xmargin) then !x.margin=xmargin
if exist(ymargin) then !y.margin=ymargin
delvarx,xarr,yarr,pic,rmap
return & end

