;+
; Project     : SOHO-CDS
;
; Name        : ROT_MAP
;
; Purpose     : rotate image contained within structure created by MAKE_MAP
;
; Category    : imaging
;
; Explanation : 
;
; Syntax      : rmap=rot_map(map,angle)
;
; Examples    :
;
; Inputs      : MAP = map structure 
;               ANGLE = angle in degrees (+ for clockwise)
;
; Opt. Inputs : None
;
; Outputs     : RMAP = map with rotated coordinates
;
; Opt. Outputs: None
;
; Keywords    : CENTER= [XC,YC] = center of rotation [def = center of image]
;                       (can also be another image map)
;             : NOREMAP = don't remap image data (just rotate coords)
;             : FULL_SIZE = expand image size to fit all rotated pixels
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 November 1996, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

function rot_map,map,angle,center=center,noremap=noremap,err=err,$
                verbose=verbose,full_size=full_size,_extra=extra

err=''

if not valid_map(map) then begin
 err='rmap=roll_map(map,angle,center=center)'
 pr_syntax,err
 if exist(map) then return,map else return,-1
endif

if not exist(angle) then begin
 err='Enter rotation angle in degrees clockwise from North'
 message,err,/cont
 if exist(map) then return,map else return,-1
endif

verbose=keyword_set(verbose)

;-- don't rotate if multiple of 360.

if (angle mod 360.) eq 0. then return,map

;-- read image and pixel arrays

nmp=n_elements(map)
for i=0,nmp-1 do begin

 unpack_map,map(i),data,xp,yp,err=err,dx=dx,dy=dy,xc=xc,yc=yc
 rcenter=[xc,yc]
 if valid_map(center) then rcenter=[get_map_xc(center),get_map_yc(center)]
 if n_elements(center) eq 2 then rcenter=float(center)
 rot_cord,xp,yp,angle,rx,ry,center=rcenter

;-- update pixel arrays, check for roll fields

 nmap=repack_map(map(i),rx,ry)
 if tag_exist(nmap,'roll') then roll=nmap.roll else begin
  roll=0.
  nmap=add_tag(nmap,roll,'roll',index='id')
 endelse
 nmap.roll=(roll+angle) mod 360.
 if not tag_exist(nmap,'roll_center') then $
  nmap=add_tag(nmap,rcenter,'roll_center',index='roll')
 nmap.roll_center=rcenter

;-- rebin image
;-- do this by regridding rotated coordinates and
;   and computing image data value in pre-rotated image by interpolation

 same=1-keyword_set(full_size)
 if (1-keyword_set(noremap)) then begin
  grid_xy,rx,ry,gx,gy,space=[dx,dy],same=same
  rot_cord,gx,gy,-angle,rx,ry,center=rcenter
  xmin=min(xp) & ymin=min(yp) & xmax=max(xp) & ymax=max(yp)
  out=where((rx lt xmin) or (rx gt xmax) or $
            (ry lt ymin) or (ry gt ymax),count)
  count=0
  if count eq n_elements(rx) then begin
   err='No data in rotated image'
   message,err,/cont
  endif else begin
   if verbose then $
    dprint,'% ROT_MAP: # of unrotated points = ',trim(string(count))
   in=where( (rx ge xmin) and (rx le xmax) and $
             (ry ge ymin) and (ry le ymax),icount)

   nmap.data=0
   rdata=interp2d(data,xp,yp,rx,ry,_extra=extra)
   if same then nmap.data=temporary(rdata) else $
    nmap=rep_tag_value(nmap,temporary(rdata),'data')
   nmap=repack_map(nmap,gx,gy)
  endelse
 endif
 rmap=merge_struct(rmap,nmap)
endfor

if not valid_map(rmap) then rmap=-1
delvarx,xp,yp,gx,gy,rx,ry,data

return,rmap & end

