;+
; Project     : SOHO-CDS
;
; Name        : SUB_MAP
;
; Purpose     : get subimage of an image map
;
; Category    : imaging
;
; Explanation : Calling this procedure with /plot will invoke PLOT_MAP
;               and sub region is selected with a cursor.
;               In this case, XRANGE and YRANGE are returned as outputs.
;               Alternatively, XRANGE and YRANGE can be input variables
;               and will be used for extraction.
;               If FOV is a valid map, then its XRANGE, YRANGE are used
;               for extraction
;
; Syntax      : smap=sub_map(map)
;
; Examples    :
;
; Inputs      : MAP = map structure created by MAKE_MAP
;
; Opt. Inputs : None
;
; Outputs     : SMAP = subimage of map
;
; Opt. Outputs: None
;
; Keywords    : NOPLOT = don't invoke PLOT_MAP for graphical selection
;               XRANGE = [x1,x2] = min/max x-coord's (data units)
;               YRANGE = [y1,y2] = min/max y-coord's
;               SUB =[x1,x2,y1,y2] = output subarray of indicies (pixel units)
;               REF = reference map index for multiple maps (if plotting)
;               FOV = reference map for inferring XRANGE, YRANGE
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 November 1997, D. Zarro, SAC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

pro sub_map,map,smap,xrange=xrange,yrange=yrange,fov=fov,$
            noplot=noplot,err=err,sub=sub,_extra=extra,ref=ref,rubber=rubber

on_error,1
err=''

if not valid_map(map,err=err) then begin
 pr_syntax,'sub_map,map,smap,[xrange=xrange,yrange=yrange])'
 return
endif

use_cursor=1
do_plot=(1-keyword_set(noplot))
use_range=((n_elements(xrange) eq 2) and (n_elements(yrange) eq 2)) or $
          (valid_map(fov))

if use_range then begin
 do_plot=0 & use_cursor=0
endif

;-- check for open window

if use_cursor and (not do_plot) then begin
 device,window=ow
 open=where(ow ne 0,count)
 if count eq 0 then do_plot=1 else wshow
endif

;-- get data coordinates to extract

nmap=n_elements(map)
if not exist(ref) then ref=0 else ref=ref < (nmap-1)

if do_plot then begin
 plot_map,map(ref),_extra=extra,err=err
 if err ne '' then return
endif

if use_cursor then begin
 region=get_sub_region(rubber=rubber)
 xrange=[region(0),region(1)]
 yrange=[region(2),region(3)]
endif else begin
 if valid_map(fov) then begin
  xrange=get_map_prop(fov(0),/xr)
  yrange=get_map_prop(fov(0),/yr)
 endif
endelse

;-- get full coordinates

xp=get_map_prop(map(ref),/xp) & yp=get_map_prop(map(ref),/yp)

;-- get pixel indicies

sub_region=[xrange(0:1),yrange(0:1)]
sub=get_map_region(xp,yp,sub_region,count=count)

if count le 2 then begin
 err='Insufficient points in sub-region'
 message,err,/cont
 return
endif

x1=sub(0)
x2=sub(1)
y1=sub(2)
y2=sub(3)

delvarx,smap
for i=0,nmap-1 do begin
 tmap=rep_tag_value(map(i),(map(i).data)(x1:x2,y1:y2),'data')
 tmap=repack_map(tmap,xp(x1:x2,y1:y2),yp(x1:x2,y1:y2))
 smap=merge_struct(smap,tmap)
endfor

return
end

