
FUNCTION DIFF_ROT, ddays, latitude, howard=howard, allen=allen, $
                   synodic=synodic, sidereal=sidereal
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:	
;       DIFF_ROT()
;
; PURPOSE:
;     Computes the differential rotation of the sun
;       
; EXPLANATION:
;       
; CALLING SEQUENCE: 
;       Result = DIFF_ROT(ddays,latitude)
;
; INPUTS:
;       DDAYS    --  number of days to rotate
;       LATITUDE --  latitude in DEGREES
;       
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       Result -- Change in longitude over ddays days in DEGREES
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       ALLEN    -- use values from Allen, Astrophysical Quantities (DEFAULT)
;       HOWARD   -- use values for small magnetic features from Howard et al.
;       SIDEREAL -- use sidereal rotation rate (DEFAULT)
;       SYNODIC  -- use synodic rotation rate
;
; CALLS:
;       None.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       
; PREVIOUS HISTORY:
;       Written T. Metcalf  June 1992
;
; MODIFICATION HISTORY:
;       Version 1, Liyun Wang, GSFC/ARC, November 17, 1994
;          Incorporated into the CDS library
;       Version 2, Zarro, GSFC, 1 July 1997 - made Howard the default
;       Version 3, Zarro, GSFC, 19 Sept 1997 - corrected Howard coeff's
;
; VERSION:
;       Version 2
;-
;
   ON_ERROR, 2
   sin2l = (SIN(FLOAT(latitude*!dtor)))^2
   sin4l = sin2l*sin2l
   IF KEYWORD_SET(allen) THEN BEGIN

;  Allen, Astrophysical Quantities

    rotation = ddays*(14.44-3.0*sin2l)
   ENDIF ELSE BEGIN

;  Small magnetic features 
;  (Howard, Harvey, and Forgach, Solar Physics, 130, 295, 1990)

    rotation = 1.e-6*ddays*(2.894-0.428*sin2l-0.37*sin4l)*24.*3600./!dtor
   ENDELSE

   IF KEYWORD_SET(synodic) THEN BEGIN
    rotation = rotation-0.9856*ddays
   ENDIF 
   RETURN, rotation
END

