;+
; Project     : SOHO - CDS     
;                   
; Name        : HEL2ARCMIN()
;               
; Purpose     : Compute position relative to sun centre from heliographic.
;               
; Explanation : Using the input heliographic coordinates of a feature,
;               calculate the position in arcmin relative to the sun centre
;               taking account of the sun's orientation (B0).  The current
;               date is assumed unless specified.  West and  North are 
;               considered positive.
;               
; Use         : IDL> print, hel2armin(ns, ew, date = dat)
;                eg  print,hel2arcmin('S34','E23')
;                or  print,hel2arcmin(-34,-23)
;    
; Inputs      : ns      -  the Heliographic latitude in degrees (can be a
;                          string with N/S first character instead of sign).
;               ew      -  the Heliographic longitude in degrees (can be a
;                          string with E/W first character instead of sign).
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the (x,y) location in arcmins relative to
;               sun disk centre.
;               
; Opt. Outputs: None
;               
; Keywords    : date    -  the date to use in the calculation of B0.
;               error   -  Output keyword containing error message;
;                          a null string is returned if no error occurs
;               soho    -  if set uses the SOHO view point rather than
;                          the Earth.  Note this functionality is
;                          duplicated by the system variable SC_VIEW,
;                          which in turn is set by the procedures
;                          USE_EARTH_VIEW or USE_SOHO_VIEW.
;
;
; Calls       : PB0R
;               ANYTIM2UTC
;
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Utilities, coordinates.
;               
; Prev. Hist. : Yohkoh routine by Hudson/Wuelser.
;
; Written     : CDS version, C D Pike, RAL, 6 Sept 93
;               
; Modified    : To use CDS time and pb0r routines, CDP, 17-May-94
;		Version 2, William Thompson, GSFC, 14 November 1994
;			Modified .DAY to .MJD
;               Version 3, 26-Feb-96, CDP
;                       Added SOHO keyword
;               Version 4, March 11, 1996, Liyun Wang, GSFC/ARC
;                  Modified such that point of view can be changed to
;                     SOHO if the env variable SC_VIEW is set to 1
;                  Added ERROR keyword
;               Allow input format of eq 'N3.4', 'E9.12'.  CDP, 12-Apr-96
;
;
; Version     : Version 5, 12-Apr-96
;-            


FUNCTION hel2arcmin, ns, ew, date=datin, soho=soho, error=error

;
;  check if wanted Soho perspective
;
   IF KEYWORD_SET(soho) OR soho_view() THEN $
      use_soho = 1 $
   ELSE $
      use_soho = 0
   error = ''

   IF datatype(ns) EQ 'STR' THEN BEGIN
      n = float(strmid(ns, 1, 100))
      IF STRUPCASE(STRMID(ns, 0, 1)) EQ 'S' THEN n = -n
      w = float(strmid(ew, 1, 100))
      IF STRUPCASE(STRMID(ew, 0, 1)) EQ 'E' THEN w = -w
   ENDIF ELSE BEGIN
      n = ns
      w = ew
   ENDELSE

;
;  convert to radians
;
   lon = w/!radeg 
   colat = (90. - n)/!radeg

;
; get system time, if required
;
   IF N_ELEMENTS(datin) EQ 0 THEN get_utc, dat  ELSE dat = anytim2utc(datin)
   IF dat.mjd EQ 0 THEN BEGIN
      error = 'Error in date format specification.'
      MESSAGE, error, /cont
      RETURN, [-99., -99.]
   ENDIF
 
;
; get B0 and solar radius
;
   angles = pb0r(dat, soho=use_soho)

;
; convert radius to arcmin
;
   sunr = angles(2)                       

;
; vect is the (x,y,z) location of the point for b0 = 0, where x is in the
; direction of Texas, y is west, and z is north. vect1 is rotated by b0. 
;   

   answer = FLTARR(2, MAX([N_ELEMENTS(colat), N_ELEMENTS(lon), $
                           N_ELEMENTS(sunr)]))

;
;  calculate the result
;
   b0 = angles(1)/!radeg
   scl = SIN(colat)
   answer(0, *) = scl * SIN(lon) * sunr
   answer(1, *) = (-scl * COS(lon) * SIN(b0) + COS(colat) * COS(b0) ) * sunr

   RETURN, answer

END
