;+
; Project     : SMM-XRP
;
; Name        : PLOT_HELIO
;
; Purpose     : Plot solar heliographic grid and limb
;
; Category    : Plotting
;
; Explanation : Uses heliographic formulae from: 
;               "Textbook on Spherical Astronomy", 
;               by W.M. Smart (see page 174).
;
; Syntax      : plot_helio,radius,bangle,roll,grid=grid,glabel=glabel
;
; Examples    :
;
; Inputs      : RADIUS = solar radius (arcsecs)
;               BANGLE = solar bangle (deg)
;
; Opt. Inputs : ROLL = spacecraft roll (deg) clockwise from solar north 
;
; Outputs     : None
;
; Opt. Outputs: LAT,LON = latitude and longitude arrays
;
; Keywords    : GRID = spacing (deg) of lat-long  (def = 15 deg);
;	       	       for grid le 0, only the solar limb is plotted
;               GLABEL = label lat-long grid with coordinate values (def = no labels)
;               NOERAS = control erasing of previously drawn plot. Setting this
;                        to 1 will cause grid to be overlaid on a drawn plot
;               COLOR  = color index for grid
;               FONT   = font index for grid label
;
; Restrictions: None
;
; Side effects: None
;
; History     : Written 22 December 1991, D. Zarro, ARC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

pro plot_helio,date,grid=grid,glabel=glabel,noeras=noeras,$
           color=color,font=font,soho=soho,roll=roll

;--  plot solar limb

err=''
tdate=anytim2utc(date,err=err)
if err ne '' then begin
 message,err,/cont
 return
endif

save_clip=!p.noclip
!p.noclip=0
pvec=pb0r(tdate,/arcsec,soho=soho)
radius=pvec(2)
if n_elements(font) eq 0 then font=-1
if n_elements(color) eq 0 then color=!p.color
circle=!dtor*findgen(366)
xlimb = radius*sin(circle)  &  ylimb = radius*cos(circle)
if keyword_set(noeras) then $
 oplot,xlimb,ylimb,linestyle=0,thick=2,noclip=0,color=color else  $
  plot,xlimb,ylimb,xstyle=5,ystyle=5,linestyle=0,thick=2,noclip=0,color=color

if n_elements(grid) eq 0 then grid=15.
if grid le 0 then return			; No grid plotted for negative grid values

;-- define latitude-longitude matrices (in degrees) at desired grid spacing

v=-90.+grid*dindgen(180./grid+1) & np=n_elements(v)
lon=rebin(v,np,np) & lat=rotate(lon,1)

;-- compute cartesian coordinates of grid points

xy=hel2arcmin(lon,lat,date=date,soho=soho)*60.
xcor=reform(xy(0,*),np,np)
ycor=reform(xy(1,*),np,np)

;-- plot latitude-longitude lines 

for i=0,np-1 do oplot,xcor(i,*),ycor(i,*),linestyle=1,noclip=0,color=color
for j=0,np-1 do oplot,xcor(*,j),ycor(*,j),linestyle=1,noclip=0,color=color

;-- label grid coordinates within current viewport

if keyword_set(glabel) then begin
 rho1=sqrt(xcor^2+ycor^2)
 within = where( (rho1 lt radius) and (abs(lon) lt 90.) and (abs(lat) lt 90.)  $
	   and (!x.crange(0) le xcor) and (xcor le !x.crange(1)) $
           and (!y.crange(0) le ycor) and (ycor le !y.crange(1)) )
 if within(0) ne -1 then begin
  for k=0,n_elements(within)-1 do begin
   clon = strcompress(nint(lon(within(k))),/remove_all)
   clat = strcompress(nint(lat(within(k))),/remove_all)
   xyouts, xcor(within(k)), ycor(within(k)), '!3 '+clat+','+clon,noclip=0,$
           color=color,font=font
  endfor
 endif
endif

!p.noclip=save_clip
return & end
