FUNCTION rot_xy, xx, yy, interval, tstart=tstart, tend=tend, date=date, $
            error=error, soho=soho, keep=keep, offlimb=offlimb, index=index,$
            _extra=extra,radius=radius
;+
; PROJECT:
;       SOHO - CDS/SUMER
;
; NAME:
;       ROT_XY()
;
; PURPOSE:
;       Get a solar rotated position for a given time interval.
;
; CATEGORY:
;       Utility, coordinates
;
; SYNTAX:
;       Result = rot_xy(xx, yy, interval [, date=date])
;       Result = rot_xy(xx, yy, tstart='96/4/19' [, tend=tend])
;
; INPUTS:
;       XX       - Solar X position (arcsecs) of the point; can be a vector
;                  (in this case YY must be a vector with same elements)
;       YY       - Solar Y position (arcsecs) of the point; can be a vector
;                  (in this case XX must be a vector with same elements)
; OPTIONAL INPUTS:
;       INTERVAL - Time interval in seconds; positive (negative) value
;                  leads to forward (backward) rotation. If INTERVAL
;                  is not given, a beginning time must be given via
;                  the TSTART keyword
;
; OUTPUTS:
;       RESULT - A (Mx2) array representing rotated positions in arcsecs,
;                RESULT(*,0) being X position and RESULT(*,1) Y position;
;                where M is number of elements in XX (and in YY).
;                If an error occurs, [-9999,-9999] will be returned.
;
;                If OFFLIMB = 1 after the call, there must be some
;                points rotated to the back of the sun. The points remain
;                visible can be determined by RESULT(INDEX,*), and
;                off-limb points will have the value of (-9999, -9999).
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORDS:
;       DATE    - Date/time at which the sun position is calculated; can
;                 be in any UTC format. If missing, current date/time is
;                 assumed.
;       TSTART  - Date/time to which XX and YY are referred; can be in
;                 any acceptable time format. Must be supplied if
;                 INTERVAL is not passed
;       TEND    - Date/time at which XX and YY will be rotated to; can be
;                 in any acceptable time format. If needed but missing,
;                 current time is assumed
;       SOHO    - Set this keyword to use SOHO view. If this keyword is
;                 missing, the point of view will depend on whether the
;                 env variable SC_VIEW is set 1 or not
;       ERROR   - Error message returned; if there is no error, a null
;                 string is returned
;       OFFLIMB - A named variable indicating whether any rotated
;                 point is off the limb (1) or not (0). When OFFLIMB
;                 is 1, the points still remaining visible (inside the limb)
;                 will be those whose indices are INDEX (below)
;       INDEX   - Indices of XX/YY which remain inside the limb after
;                 rotation. When OFFLIMB becomes 1, the number of
;                 INDEX will be smaller than that of XX or YY. If no
;                 point remains inside the limb, INDEX is set to -1
;       KEEP    - keep same epoch DATE when rotating
;       
;       RADIUS  - solar radius value [output in arcmin units]
;
; COMMON:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; HISTORY:
;       Version 1, March 18, 1996, Liyun Wang, NASA/GSFC. Written
;       Version 2, April 16, 1996, Liyun Wang, NASA/GSFC
;          Added SOHO keyword
;       Version 3, April 18, 1996, Liyun Wang, NASA/GSFC
;          Added keywords TSTART and TEND
;          Made INTERVAL an optional input parameter if TSTART is set
;       Version 4, July 15, 1996, Zarro, NASA/GSFC
;          Added KEEP keyword
;       Version 5, July 31, 1996, Liyun Wang, NASA/GSFC
;          Modified such that no calculation is done if interval is zero
;       Version 6, March 11, 1997, Liyun Wang, NASA/GSFC
;          Added OFFLIMB keyword
;       Version 7, May 16, 1997, Liyun Wang, NASA/GSFC
;          Added INDEX keyword
;          Fixed a bug feeding off-limb points to HEL2ARCMIN
;       Version 8, June 10, 1997, Liyun Wang, NASA/GSFC
;          Properly handled off-limb points
;       Version 9, June 10, 1997, Liyun Wang, NASA/GSFC
;          Properly handled off-limb points
;       Version 10, July 1 1997 , Zarro SAC/GSFC -- added _EXTRA
;       Version 11, Sept 10, 1997, Zarro SAC/GSFC 
;          Added RADIUS output keyword
;       Version 12, September 19, 1997, Liyun Wang, NASA/GSFC
;          Fixed a bug that caused INDEX undefined when no rotation is needed
;
; CONTACT:
;       Liyun Wang, NASA/GSFC (Liyun.Wang.1@gsfc.nasa.gov)
;-
;
   ON_ERROR, 2
   error = ''
   offlimb = 0
   dum=[-9999,-9999]
   IF N_PARAMS() LT 2 THEN BEGIN
      error = 'Syntax: a = rot_xy(x, y, interval)'
      MESSAGE, error, /cont
      RETURN, dum
   ENDIF
   IF N_ELEMENTS(xx) NE N_ELEMENTS(yy) THEN BEGIN
      error = 'XX and YY must be scalar or vector with the same elements'
      MESSAGE, error, /cont
      RETURN,dum
   ENDIF

   IF N_ELEMENTS(interval) EQ 0 AND N_ELEMENTS(tstart) EQ 0 THEN BEGIN
      error = 'You need to specify a time interval'
      MESSAGE, error, /cont
      RETURN,dum
   ENDIF

;---------------------------------------------------------------------------
;  Determine time interval
;---------------------------------------------------------------------------
   IF N_ELEMENTS(interval) EQ 0 THEN BEGIN
      date = anytim2utc(tstart)
      IF N_ELEMENTS(tend) EQ 0 THEN get_utc, tend
      interval = utc2tai(anytim2utc(tend))-utc2tai(date)
   ENDIF

   temp = TRANSPOSE(arcmin2hel(xx/60.0, yy/60.0, date=date, off=off, $
                               radius=radius,/no_copy))
   
;---------------------------------------------------------------------------
;  If time interval is zero, bail out
;---------------------------------------------------------------------------
   IF interval EQ 0.d0 THEN BEGIN
      MESSAGE, 'Zero time interval. No need to rotate.', /cont
      index = WHERE(temp(*, 0) NE -9999.0)
      RETURN, [[xx], [yy]]
   ENDIF

   IF N_ELEMENTS(date) EQ 0 THEN get_utc, date
   tmp = anytim2utc(date, err=error)
   IF error NE '' THEN BEGIN
      MESSAGE, error, /cont
      RETURN,dum
   ENDIF

   IF KEYWORD_SET(soho) THEN BEGIN
      old_view = GETENV('SC_VIEW')
      IF old_view EQ '' THEN old_view = '0'
      use_soho_view
   ENDIF

   ii = WHERE(off EQ 1, count)
   IF count EQ N_ELEMENTS(xx)  THEN BEGIN
      error = 'All initial points are off the limb; cannot rotate!'
      temp(*, *) = -9999.0
      offlimb = 1
      index = -1
      RETURN, temp
   ENDIF

   IF count NE 0 THEN BEGIN
      temp(ii, 0) = 1.0
      temp(ii, 1) = -9999.0
   ENDIF

   ddays =float(interval)/86400.0
   temp(*, 1) = temp(*, 1)+diff_rot(ddays, temp(*, 0), /synodic,_extra=extra)
   in = WHERE(ABS(temp(*, 1)) LE 90.0, count)
   IF count EQ 0 THEN BEGIN
      temp(*,*) = -9999.0
      index = -1
      RETURN, temp
   ENDIF ELSE IF count NE N_ELEMENTS(temp(*, 1)) THEN BEGIN
      out = WHERE(ABS(temp(*, 1)) GT 90.0, count_out)
      offlimb = 1
   ENDIF

   IF NOT KEYWORD_SET(keep) THEN tmp.time = tmp.time+DOUBLE(interval)

   IF offlimb EQ 0 THEN BEGIN
      temp = 60.0*TRANSPOSE(hel2arcmin([temp(*, 0)], [temp(*, 1)], date=tmp))
   ENDIF ELSE BEGIN
      IF in(0) GE 0 THEN BEGIN
         subtemp = temp(in, *)
         subtemp = 60.0*TRANSPOSE(hel2arcmin([subtemp(*, 0)], [subtemp(*, 1)],$
                                             date=tmp))
         temp(in, *) = subtemp
         temp(out, *) = -9999.0
      ENDIF ELSE temp(*,*) = -9999.0
   ENDELSE

   IF KEYWORD_SET(soho) THEN setenv,'SC_VIEW='+old_view
   index = WHERE(temp(*,0) NE -9999.0)
   RETURN, temp
END

