;----------------------------------------------------------------------
; Document name: grep.pro
; Created by:    Liyun Wang, GSFC/ARC, December 15, 1994
;
; Last Modified: Wed Mar 12 18:06:39 1997 (LWANG@sumop1.nascom.nasa.gov)
;----------------------------------------------------------------------
;
FUNCTION GREP, str, strarray, sensitive=sensitive, exact=exact, index=index,$
               exclude=exclude
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       GREP()
;
; PURPOSE:
;       Search for string through a string array (cf grep in Perl)
;
; EXPLANATION:
;
; CALLING SEQUENCE:
;       Result = GREP(str, strarray)
;
; INPUTS:
;       STR      - Target string to be searched for
;       STRARRAY - String array over which the string is seached
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result   - String array of elements from STRARRAY which contains
;                  the searched string STR. A null string will be returned if
;                  no elements in STRARRAY match STR.
;
; OPTIONAL OUTPUTS:
;       INDEX    - Index of elements in STRARRAY which match the target string
;
; KEYWORD PARAMETERS:
;       SENSITIVE - Make the search pattern case sensitive, if set
;       EXACT     - Requires exact match (instead of substring match)
;       EXCLUDE   - Set this keyword to return string array that does 
;                   dot contain string STR
;
; CALLS:
;       DATATYPE
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;
; PREVIOUS HISTORY:
;       Written December 15, 1994, Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Version 2, Liyun Wang, GSFC/ARC, January 20, 1995
;          Added the INDEX keyword
;       Version 3, July 26, 1995, Liyun Wang, GSFC/ARC
;          Made it return a null string if error occurs (instead of
;             stopping the program.
;       Version 4, April 7, 1996, Liyun Wang, GSFC/ARC
;          Used intrinsic function WHERE; thanks to Richard Schwartz
;       Version 5, March 12, 1997, Liyun Wang, NASA/GSFC
;          Added the EXCLUDE keyword
;
; VERSION:
;       Version 5, March 12, 1997
;-
;
   ON_ERROR, 2

   IF datatype(str) NE 'STR' OR datatype(strarray) NE 'STR' THEN BEGIN
      MESSAGE, 'Input parameter must be of string type.', /cont
      RETURN, ''
   ENDIF

   aa = strarray
   a = str
   IF NOT KEYWORD_SET(sensitive) THEN BEGIN
      aa = STRUPCASE(aa)
      a = STRUPCASE(a)
   ENDIF

   IF KEYWORD_SET(exclude) THEN BEGIN 
      IF KEYWORD_SET(exact) THEN $
         index = WHERE(aa NE a) $
      ELSE $
         index = WHERE(STRPOS(aa, a) EQ -1)
   ENDIF ELSE BEGIN 
      IF KEYWORD_SET(exact) THEN $
         index = WHERE(aa EQ a) $
      ELSE $
         index = WHERE(STRPOS(aa, a) NE -1)
   ENDELSE

   IF index(0) GE 0 THEN BEGIN
      IF N_ELEMENTS(index) EQ 1 THEN BEGIN
;---------------------------------------------------------------------------
;        Make sure the returned values are of scalar type
;---------------------------------------------------------------------------
         index = index(0)
         RETURN, (strarray(index))(0)
      ENDIF ELSE $
         RETURN, strarray(index)
   ENDIF ELSE BEGIN
      index = -1
      RETURN, ''
   ENDELSE
END

;----------------------------------------------------------------------
; End of 'grep.pro'.
;----------------------------------------------------------------------
