;+
; Project     :	SDAC
;
; Name        :	ADD_TAG
;
; Purpose     :	add a tag to a structure
;
; Explanation :	
;
; Use         : NEW_STRUCT=ADD_TAG(STRUCT,TAG,TAG_NAME)
;
; Inputs      :	STRUCT = input structure (array or scalar)
;             : TAG_VALUE = tag variable to add 
;             : TAG_NAME = tag name 
;
; Opt. Inputs :	None.
;
; Outputs     :	NEW_STRUCT = new structure
;
; Opt. Outputs:	None.
;
; Keywords    :	NAME = new name for structure
;               INDEX = index or tag name where to append new tag [def = last]
;               ERR   = error message [blank if ok]
;               DUPLICATE = set to allow duplicate tag names
;		TOP_LEVEL = If set, then only the top level is searched to
;			    determine if the tag already exists.
;               NO_COPY = do not make copy of input TAG variable
;                         (it will be destroyed after input)
;
; Restrictions:	Cannot add more than one tag at a time
;
; Side effects:	None.
;
; Category    :	Structure handling
;
; Prev. Hist. :	None.
;
; Written     :	Dominic Zarro (ARC)
;               
; Version     :	Version 1.0, 7 November 1994 -- written
;               Version 2.0, 16 June 1996    -- cleaned up
;		Version 3, 11-Aug-1997, William Thompson, GSFC
;			Added keyword TOP_LEVEL
;-
;----------------------------------------------------------------------------

function add_tag_err,val,err
mess='syntax --> NEW_STRUCT=ADD_TAG(STRUCT,TAG_VALUE,TAG_NAME)'
err=mess & message,err,/cont
if exist(val) then return,val else return,0
end

;----------------------------------------------------------------------------

function add_tag,struct,tag_value,tag_name,name=name,index=index,err=err,$
                 duplicate=duplicate,top_level=top_level,no_copy=no_copy

on_error,1

err=''
if not exist(tag_value) then return,add_tag_err(struct,err)

;-- just tag value and name entered

new_struct=-1
if  (n_params() eq 2) then begin
 if (datatype(tag_value) eq 'STR') then begin
  if (trim(tag_value) ne '') then begin
   if datatype(struct) ne 'STC' then begin
    new_struct=create_struct(tag_value,struct)
    goto,done
   endif
  endif
 endif 
 return,add_tag_err(struct,err)
endif

;-- no tag name was entered

if datatype(tag_name) eq 'STR' then tname=tag_name else tname=''
tname=strtrim(tname(0),2)
if tname eq '' then begin
 err='tag name must be non-blank string'
 message,err,/cont
 if exist(struct) then return,struct else return,0
endif
tname=strupcase(tname)

if n_elements(tname) ne 1 then begin
 err='restricted to adding one tag at a time'
 message,err,/cont
 if exist(struct) then return,struct else return,0
endif

;-- input structure undefined

if (datatype(struct) ne 'STC') and exist(tag_value) then begin
 new_struct=create_struct(tname,tag_value)
 goto,done
endif

;-- does tag already exist

if not keyword_set(duplicate) then begin
 if datatype(struct) eq 'STC' then begin
  if tag_exist(struct,tname,top_level=top_level) then begin
;   err=tname+' tag already exists'
;   message,err,/cont
   return,struct
  endif
 endif
endif

;-- determine location of added tag

tags=tag_names(struct)
ntags=n_elements(tags)
aindex=ntags-1
if exist(index) then begin
 if datatype(index) eq 'STR' then begin
  ilook=where(strupcase(trim(index)) eq tags,icount)
  if icount gt 0 then aindex=ilook(0)
 endif else aindex=index(0)
endif
aindex= -1 > aindex < (ntags-1)
no_copy=keyword_set(no_copy)

delvarx,new_struct
for k=0,n_elements(struct)-1 do begin
 delvarx,temp_struct
 if aindex eq (ntags-1) then begin
  if no_copy then $
   temp_struct=create_struct(struct(k),tname,temporary(tag_value)) else $
    temp_struct=create_struct(struct(k),tname,tag_value)
 endif else begin
  for i=0,ntags-1 do begin
   prepend=(i eq 0) and (aindex eq -1)
   if prepend then temp_struct=create_struct(tname,tag_value)
   if exist(temp_struct) then $
    temp_struct=create_struct(temporary(temp_struct),tags(i),struct(k).(i)) else $
     temp_struct=create_struct(tags(i),struct(k).(i))
   append=(i eq aindex)
   if append then begin
    if no_copy then $
     temp_struct=create_struct(temporary(temp_struct),tname,temporary(tag_value)) else $
      temp_struct=create_struct(temporary(temp_struct),tname,tag_value)
   endif
  endfor
 endelse
 if not exist(new_struct) then new_struct=temporary(temp_struct) else $
  new_struct=merge_struct(temporary(new_struct),temporary(temp_struct))
endfor

;--  name new structure

done: new_struct=rep_struct_name(temporary(new_struct),name)

return,new_struct & end


