;+
; Project     :	SDAC
;
; Name        :	REP_TAG_VALUE
;
; Purpose     :	replace tag value in a structure
;
; Explanation :	Useful for avoiding name definition
;               conflicts when changing structure fields
;
; Use         : NEW_STRUCT=REP_TAG_VALUE(STRUCT,TAG_VALUE,TAG_NAME)
;
; Inputs      :	STRUCT = input structure
;             : TAG_VALUE = value to give tag
;             : TAG_NAME = string tag name to modify (or index value)
;
; Opt. Inputs :	None.
;
; Outputs     :	NEW_STRUCT = new structure
;
; Opt. Outputs:	None.
;
; Keywords    :	NAME = name to give output structure [def = ANON].
;               RECURSE = search and replace recursively
;               NO_COPY = do not make copy of input TAG_VALUE
;                         (it will be destroyed after input)
;
; Calls       : CONCAT_STRUCT
;
; Common      :	None.
;
; Restrictions:	Restricted to changing single tags at a time
;
; Side effects:	None.
;
; Category    :	Structure handling
;
; Prev. Hist. :	None.
;
; Written     :	Dominic Zarro (ARC)
;
; Version     :	Version 1.0, 4 May 1995
;               Version 2.0, 23 March 1998 -- added nested search
;-


function rep_tag_value,struct,tag_value,tag_name,name=name,err=err,$
                       quiet=quiet,recurse=recurse,no_copy=no_copy

on_error,1
err=''

verbose=1-keyword_set(quiet)
recurse=keyword_set(recurse)

if (datatype(struct) ne 'STC') or (n_elements(tag_value) eq 0) or $
   (n_elements(tag_name) eq 0) then begin
 pr_syntax,'new_struct=rep_tag_value(struct,tag_value,tag_name)'
 if exist(struct) then return,struct else return,0
endif

if n_elements(tag_name) ne 1 then begin
 message,'restricted to replacing one tag at a time',/cont
 if exist(struct) then return,struct else return,0
endif

;- do recursive search and append tag if not present

tags=tag_names(struct) & ntags=n_elements(tags)
if datatype(tag_name) eq 'STR' then begin
 tag_index=(where(strupcase(tag_name) eq tags,cnt))(0)
 if (cnt eq 0) then begin
  if recurse then begin
   for k=0,ntags-1 do begin
    if datatype(struct.(k)) eq 'STC' then begin
     tstruct=rep_tag_value(struct.(k),tag_value,tag_name,err=err,$
             no_copy=no_copy,/quiet)
     if err eq '' then begin
      new_struct=rep_tag_value(struct,tstruct,tags(k),name=name,/no_copy)
      return,new_struct
     endif
    endif
   endfor
  endif
  err='tag name < '+tag_name+' > not found'
  if verbose then message,err,/cont
  if exist(struct) then return,struct else return,0
 endif else str_tag_name=tag_name
endif else begin
 tag_index=tag_name 
 if (tag_index lt 0) or (tag_index gt (ntags-1)) then begin
  err='index out of range'
  if verbose then message,err,/cont
  return,struct
 endif
 str_tag_name=tags(tag_index)
endelse

for k=0,n_elements(struct)-1 do begin
 temp_struct=rem_tag(struct(k),str_tag_name)
 err=''
 temp_struct=add_tag(temporary(temp_struct),tag_value,str_tag_name,$
                     index=tag_index-1,err=err,no_copy=no_copy)
 if err ne '' then temp_struct=struct(k) 
 if not exist(new_struct) then new_struct=temporary(temp_struct) else $
  new_struct=merge_struct(temporary(new_struct),temporary(temp_struct))
endfor

new_struct=rep_struct_name(temporary(new_struct),name)

return,new_struct & end
