function str_subset, instructs, taglist, include=include, exclude=exclude, $
                     version=version
;+
;   Name: str_subset 
;
;   Purpose: generate a (smaller) structure with a subset of selected tags
;
;   Input Parameters:
;     instructs - the parent structure(s) 
;     taglist   - list of tag names to include (or exclude) in output structure
;                 (string array, comma delimited string, or template structure)
;
;   Keyword Parameters:
;     version   - optional structure version number to include (float)
;     exclude   - if set, TAGLIST is list of tags to EXCLUDE
;
;   Calling Sequence:
;      smallstr=str_subset(bigstruct, taglist [,/exclude] [,version= nn.] )
;
;   Calling Examples:
;   IDL> new=str_subset(eitstruct,'filename,wavelnth,filter,object,date_obs') 
;   IDL> help,new,/str
;        ** Structure MS_232241394011, 5 tags, length=72:
;        DATE_OBS        STRING    '1997-04-30T00:20:22.000Z'
;        FILENAME        STRING    'efr19970430.002022'
;        OBJECT          STRING    'full FOV'
;        WAVELNTH        INT            195
;        FILTER          STRING    'Clear'
;
;   History:
;      6-may-1997 - S.L.Freeland 
;
;   Calls:
;      data_chk, str2arr, make_str, where_arr, fmt_tag, uniq, is_member
;-

if (1- data_chk(instructs,/struct)) then begin
   message,/info,"IDL> smallstr=str_subset(bigstruct, taglist)
   return,''
endif 

retval=instructs

; figure tag list (string, string array or structure template)
case 1 of
    data_chk(taglist,/struct): stags=tag_names(taglist)
    data_chk(taglist,/scalar,/string): stags=str2arr(taglist)
    data_chk(taglist,/string): stags=taglist
    else: begin
       message,/info,"IDL> smallstr=str_subset(bigstruct, taglist)
      return,instructs
    endelse
endcase

stags=strtrim(strupcase(stags),2)
stags=stags(uniq(stags,sort(stags)))
alltags=tag_names(instructs)

; ---------- match taglist to template --------------
which=where_arr(alltags, stags, count, notequal=keyword_set(exclude))

; add VERSION tag on request (possibly overwrite existing value)
vtag=(['',',version:0.'])( (n_elements(version) ne 0) and $
                           (1-is_member('VERSION',stags))  ) 

if count gt 0 then begin
;  ----------- build structure via make structure -------------
   str='{dummy' 
   for i=0,count-1 do str=str + ',' + alltags(which(i)) + ':' + $
      fmt_tag(size(instructs(0).(which(i))))
   newstruct=make_str(str+vtag+'}')

;  ------- define and fill output structure -------------
   retval=replicate(newstruct,n_elements(instructs))
   for i=0,count-1 do retval.(i)=instructs.(which(i))
   if keyword_set(version) then retval.VERSION=version

endif else message,/info,"No tags matching desired list...."

return,retval
end

