pro make_mirror, site, remote_dir, local_dir, 				  $
		packages=packages, comments=comments,			  $
		remote_user=remote_user, remote_password=remote_password, $ 
		mirror_file=mirror_file,mode_copy=mode_copy,      	  $
		use_timelocal=use_timelocal, _extra=_extra,               $
                user=user, group=group, $
                max_delete_files=max_delete_files, $
                max_delete_dirs=max_delete_dirs
;+
;   Name: make_mirror
;
;   Purpose: build a mirror file
;
;   Input Parameters:
;      site       - remote site / ip
;      remote_dir - list of remote_dirs to mirror
;      local_dir  - list of local dirs  (destination) 
;
;   Keyword Parameters:
;      remote_user -   (default = anonymous)
;      remote_password (default = user@local_host) 
;      packages - if passed, strarray containing package names (one per remote dir)
;      comments - is passed, strarray containing comments (one per remote dir)
;      mirror_file - output mirror file name w/path (default= "mk_mirror" in current)
;      XXX=NN      - anyother where XXX=anyother mirror keword
;                    and             NN=associated value
;   
;   Calling Sequence:
;      make_mirror, siteip, remote_dir, local_dir [,mirror_file='filename', $
;			    packages=[package_names]
;
;   Calling Example:
;      IDL> <run routine to define remote and local directory lists>      
;      IDL> make_mirror,'sohoftp.nascom.nasa.gov',remotedirs, localdirs, mirror_file='SSW.mirror'
;      IDL> make_mirror,ip,remote,local,MAX_DAYS=10  ; recent 10 days only
;                                                    ; (via keyword inherit)  
;
;   18-march-1996 (S.L.Freeland) - originally for remote SSW install/upgrades
;   21-Oct-96	RDB	Added comments keyword
;   21-Oct-96   S.L.Freeland - change compress_pat to compress_patt
;   23-Oct-96   S.L.Freeland - add /MODE_COPY keyword
;   12-nov-96   S.L.Freeland - add USE_TIMELOCAL (default=false)
;   31-mar-97   S.L.Freeland - add USER and GROUP keywords
;   22-may-97   S.L.Freeland - add MAX_DELETE_FILES  and MAX_DELETE_DIRS
;    4-feb-98   S.L.Freeland - init MAX_DELETE_FILES MAX_DELETE_DIRS if 
;                              undefined (set to 10%, the mirror default)
;   20-mar-98   S.L.Freeland - enabled the keyword inheritance part
;
;   Restrictions:
;      must supply siteip and remote & local dirs (minimum)
;      assume one-one mapping between remote and local
;-
if not data_chk(site,/string) then begin
   message,/info,"Need to specify remote site..."
   return
endif

npacks=n_elements(remote_dir)

case 1 of 
   data_chk(remote_dir,/string) and data_chk(local_dir,/string): begin
      if npacks ne n_elements(local_dir) then begin
         message,/info,"number of remote and local dirs must match..."
         return
      endif 
   endcase
   else: begin
      message,/info,"Need to specify remote and local directory lists..."
      return
   endcase
endcase

; set some default field values
compress_patt=''
if n_elements(max_delete_files) eq 0 then max_delete_files='10%'
if n_elements(max_delete_dirs) eq 0 then max_delete_dirs='10%'

mcomment='Generated by make_mirror at: ' + systime()
comment=''
if not keyword_set(comments) then comment = mcomment
package= str_replace(remote_dir,'/','_')	
if not keyword_set(remote_user) then begin
   remote_user='anonymous'
   remote_password=get_user()+'@'+get_host()
endif

mode_copy=(['false','true'])(keyword_set(mode_copy)) ; use mirror default=false
use_timelocal=(['false','true'])(keyword_set(use_timelocal)) ; use mirror default=false

if not keyword_set(user) then user=get_user()
if not keyword_set(group) then group=''

; define minimal mirror fields
; ------------------ offer template file option ?? --------
packstr={package:'',		                $
         comment:'',				$
	 site:site,		                $
	 remote_user:'',		        $
         remote_password:'',	                $
	 remote_dir:'',		                $
         local_dir:'',		                $
         mode_copy:'',				$
         use_timelocal:'',			$
         user:'',				$
         group:'',				$
         max_delete_files:'',                   $
         max_delete_dirs:'',                    $
         compress_patt:''}          
; ------------------ offer template file option ?? --------

allpacks=replicate(packstr,npacks)		; one per package
tags=strlowcase(tag_names(packstr))		; info
ntags=n_tags(packstr)				; info

for i=0,n_tags(packstr)-1 do begin		; assign tag values
   exe=execute('val=' + tags(i))
   exe=execute('allpacks.(i)=tags(i)+"="+val')
endfor

if n_elements(packages) eq npacks then allpacks.package='package='+packages
if n_elements(comments) eq npacks then allpacks.comment='comment='+comments
if not keyword_set(mirror_file) then mirror_file='mk_mirror'

if data_chk(_extra,/struct) then begin
   tnames=strlowcase(tag_names(_extra))
   for i=0,n_tags(_extra)-1 do allpacks=$
	add_tag(allpacks,tnames(i)+'='+strtrim(_extra.(i),2),tnames(i))
endif  

ntags=n_tags(allpacks(0))

; write output mirror file 
form='(' + strtrim(npacks,2) + '(1h#,/,a,/,'+ strtrim(ntags-1,2) + '(3x,a,/)))'
file_append,mirror_file,'# '+mcomment,/new
file_append,mirror_file,string(allpacks,format=form)	;, /new
message,/info,"Wrote mirror file: " + mirror_file

return
end
