;+
; Project     : SOHO-CDS
;
; Name        : SMART_FTP
;
; Purpose     : A wrapper around FTP_COPY
;
; Category    : planning
;
; Explanation :
;
; Syntax      : smart_ftp,server,get_files,get_dir
;
; Examples    :
;
; Inputs      : SERVER = server to retrieve from
;               GET_FILES = filenames to retrieve
;               GET_DIR = directory to retrieve from
;
; Opt. Inputs : None
;
; Outputs     : 
;
; Opt. Outputs: 
;
; Keywords    : OUTDIR = output directory for file [def = current]
;               FILES = found filenames
;               ERR = error string
;               COUNT = no of files copied
;               RETRY = # times to retry if server is down
;               KILL  = kill any zombie processes
;               QUIET = turn off output messages
;
; Common      : None
;
; Restrictions: Unix only 
;
; Side effects: None
;
; History     : Written 14 May 1998 D. Zarro, SAC/GSFC
;
; Contact     : dzarro@solar.stanford.edu
;-

pro smart_ftp,server,get_files,get_dir,outdir=outdir,count=count,err=err,$
    retry=retry,quiet=quiet,kill=kill,files=files,_extra=extra

on_error,1
err=''
count=0
files=''
quiet=keyword_set(quiet)
loud=1-quiet

if (datatype(get_files) ne 'STR') or (datatype(get_dir) ne 'STR') or $
   (datatype(server) ne 'STR') then begin
 pr_syntax,'smart_ftp,server,get_files,get_dir'
 return
endif

;-- check write access

if datatype(outdir) ne 'STR' then put_dir=curdir() else put_dir=outdir
if not test_dir(put_dir,quiet=quiet,err=err) then return
 
;-- call FTP_COPY

if not exist(retry) then retry=1 else retry=nint(retry) > 1
icount=0 & status=0
nfiles=n_elements(get_files)
repeat begin 
 if keyword_set(kill) then kill_job,server
 ftp_copy,get_files,server,subdir=get_dir,/get,/anon,outdir=put_dir,$
  status=status,_extra=extra
 icount=icount+1
 chk_stat=where(status eq 1,scount)
endrep until ((scount eq nfiles) or (icount eq retry))

tcount=0
for i=0,nfiles-1 do begin
 tfiles=loc_file(get_files(i),path=put_dir,count=tcount)
 if (tcount gt 0) then begin
  if trim(files(0)) eq '' then files=tfiles else files=[files,tfiles]
 endif
endfor
if trim(files(0)) ne '' then count=n_elements(files)

ftp_scr=concat_dir(getenv('HOME'),'ftp_copy.ftp')
rm_file,ftp_scr

return & end

