;+
; Project     : SOHO - CDS
;
; Name        : TEST_OPEN
;
; Purpose     : Test open a file to determine existence and/or write access
;
; Explanation :
;       Uses OPENR or OPENW (if /WRITE is set) to return status of
;       input file. If input file is not given, then TEST_OPEN will
;       test if the current directory is writeable.
; Use         : 
;	OK=TEST_OPEN(FILE)             ;test for existence
;	OK=TEST_OPEN(FILE,/WRITE)      ;test for write access
; Inputs      : 
;	FILE  = file to test
; Opt. Inputs : 
;	NAME = String containing the name of the procedure.
; Outputs     : 
;	OK   = logical 1 for existence and readable
;                   or 0 for nonexistent and/or not writeable (if /WRITE)
; Opt. Outputs: 
;	None.
; Keywords    : 
;       WRITE = If set, then test for write access.
;       QUIET = set to keep quiet
;       NODIR   = do not test if input is a directory (for speed)
;       ERR     = error string
; Calls       : 
;	None
; Common      : 
;	None.
; Restrictions: 
;	None.
; Side effects: 
;	None.
; Category    : 
;	Utilities, Operating_system.
; Written     : 
;	Dominic Zarro, GSFC, 1993.
; Version     : 
;	Version 1, 1 August 1993.
;       Version 2, 15 March 1995, modified, Zarro 
;        --  added check for directory input
;       Version 3, 3 September 1996, Zarro
;        -- added VMS check for non-existant file input
;-

function test_open,file,write=write,quiet=quiet,nodir=nodir,err=err
err=''
ok=0 
if datatype(file) ne 'STR' then is_dir=1 else begin
 if not keyword_set(nodir) then is_dir=chk_dir(file,out) else is_dir=0
endelse
vms=os_family() eq 'vms'

if is_dir then begin
 del=1 & app=0
 if keyword_set(write) then temp='test.garbage' else temp=''
 if datatype(out) eq 'STR'  then temp=concat_dir(out,temp)
endif else begin
 del=0 & app=1 & temp=file 
endelse

;-- if VMS, try to rename file to itself to test for write priviledge.
;   This avoids possible file locking.

if keyword_set(write) then begin
 not_found=1
 if vms and (not del) then begin
  cmd='rename/nolog/noconfirm '
  spawn,cmd+' '+temp+' '+temp,out,count=err,/nowait,/nolog,/nocli
  not_found=strpos(out(0),'SEARCHFAIL') gt -1
 endif
 if not_found then openw,lun,temp,/get_lun,delete=del,append=app,error=error else error=0
endif else openr,lun,temp,/get_lun,error=error

ok=(error eq 0)

if ok and exist(lun) then begin 
 handle=fstat(lun)
 if not is_dir then ofile=handle.name
endif

if exist(lun) then begin
 free_lun,lun & close,lun 
endif

quiet=keyword_set(quiet)
if (not ok) and keyword_set(write) then begin
 err='Denied write permission to '+file
 if not quiet then  message,err,/contin
endif

return,ok & end

