 pro hrebin, oldim, oldhd, newim, newhd, newx, newy, $
            SAMPLE=sample, OUTSIZE = outsize
;+
; NAME:
;	HREBIN
; PURPOSE:
;	Expand or contract a FITS image using REBIN and update the header array 
; EXPLANATION:
;	The size of the new image must be an integer factor of 
;	the original image size.    User can either overwrite the input array,
;	or write to new variables.
;
; CALLING SEQUENCE:
;	hrebin, oldhd        ;Special calling sequence to just update header
;	hrebin, oldim, oldhd, [ newim, newhd, newx, newy, OUTSIZE = ,SAMPLE = ]
;
; INPUTS:
;	OLDIM - the original image array
;	OLDHD - the original image FITS header, string array
;
; OPTIONAL INPUTS:
;	NEWX - size of the new image in the X direction, integer scalar
;	NEWY - size of the new image in the Y direction, integer scalar
;		HREBIN will prompt for NEWX and NEWY if not supplied
;
; OPTIONAL OUTPUTS:
;	NEWIM - the image after expansion or contraction with REBIN
;	NEWHD - header for newim containing updated astrometry info
;		If output parameters are not supplied, the program
;		will modify the input parameters OLDIM and OLDHD
;		to contain the new array and updated header.
;
; OPTIONAL INPUT KEYWORDS:
;	SAMPLE - Expansion or contraction is done using REBIN which uses 
;		bilinear interpolation when magnifying and boxaveraging when 
;		minifying.
;		If the SAMPLE keyword is supplied and non-zero, then nearest 
;		neighbor sampling is used in both cases.   
;
;	OUTSIZE - Two element integer vector which can be used instead of the
;		NEWX and NEWY parameters to specify the output image dimensions
;
; PROCEDURE:
;	The parameters BSCALE, NAXIS1, NAXIS2, CRPIX1, and CRPIX2 and the CD 
;	(or CDELT) parameters are updated for the new FITS header.
;
; EXAMPLE:
;	Compress a 2048 x 2048 image array IM, with header FITS HDR, to a 
;	512 x 512 array.   Overwrite the input variables with the compressed 
;	image and header.
;
;	IDL> hrebin, im, hdr, OUT = [512,512]
;
; PROCEDURES USED:
;	SXPAR(), SXADDHIST, CHECK_FITS
;
; MODIFICATION HISTORY:
;	Written, December 1990  W. Landsman, ST System Corp.
;	Update CD1_1 keywords   W. Landsman   November 1992
;	Check for a GSSS header   W. Landsman  June 1994
;	Update BSCALE even if no astrometry present   W. Landsman  May 1997
;- 
 On_error,2

 npar = N_params()	;Check # of parameters
 if (npar EQ 3) or (npar EQ 5) or (npar EQ 0) then begin
   print,'Syntax: HREBIN, oldim, oldhd,[ newim, newhd, OUTSIZE=, /SAMPLE ]
   return
 endif

 if not keyword_set(SAMPLE) then sample = 0

; If only 1 parameter is supplied, then assume it is a FITS header

 if ( npar EQ 1 ) then begin           

        zparcheck, 'HREBIN', oldim, 1, 7, 1, 'Image header'
	oldhd = oldim
        xsize = sxpar( oldhd,'NAXIS1' )
        ysize = sxpar( oldhd,'NAXIS2' )

 endif else begin 

     check_FITS, oldim, oldhd, dimen, /NOTYPE
     if !ERR EQ -1 then message,'ERROR - Invalid image or FITS header array'
     if N_elements(dimen) NE 2 then message, $
               'ERROR - Input image array must be 2-dimensional'
      xsize = dimen(0)  &  ysize = dimen(1)
 endelse

 if ( npar LT 6 ) then begin

    if ( N_elements(OUTSIZE) NE 2 ) then begin
    tit = !MSG_PREFIX + 'HREBIN: '
    print, tit, 'Original array size is '+ strn(xsize) + ' by ' + strn(ysize)
    read, tit + 'Enter size of new image in the X direction: ',newx
    read, tit + 'Enter size of new image in the Y direction: ',newy
  endif else begin
     newx = outsize(0)
     newy = outsize(1)
   endelse 
 
 endif

;  If an image array supplied then apply the IDL REBIN function

 if npar GT 1 then begin
   if npar GT 2 then newim = rebin( oldim, newx, newy, SAMPLE=sample) $
      	        else oldim = rebin( oldim, newx, newy, SAMPLE=sample)
 endif

 if ( sample GT 0 ) then type = ' Nearest Neighbor Approximation' else begin
          if ( newx LT xsize ) then type = ' Box Averaging' else $
                                    type = ' Bilinear Interpolation'
 endelse

 newhd = oldhd
 sxaddpar, newhd, 'NAXIS1', fix(newx)
 sxaddpar, newhd, 'NAXIS2', fix(newy)
 label = 'HREBIN: '+ strmid( systime(),4,20 )
 sxaddpar,newhd,'history',label + ' Original Image Size Was '+ $
         strn(xsize) +' by ' +  strn(ysize) 
 if ( npar GT 1 ) then sxaddpar,newhd,'history',label+type

; Update astrometry info if it exists

 extast, newhd, astr, noparams
 if noparams GE 0 then begin
 if strmid(astr.ctype(0),5,3) EQ 'GSS' then begin
	gsss_stdast, newhd
	extast, newhd, astr, noparams
 endif

 xratio = float(newx) / xsize	;Expansion or contraction in X
 yratio = float(newy) / ysize	;Expansion or contraction in Y
 pix_ratio = xratio*yratio	;Ratio of pixel areas

; Correct the position of the reference pixel.   Note that CRPIX values are
; given in FORTRAN (first pixel is (1,1)) convention

 crpix = astr.crpix
 sxaddpar, newhd, 'CRPIX1', (crpix(0)-0.5)*xratio + 0.5, FORMAT='(E14.7)'
 sxaddpar, newhd, 'CRPIX2', (crpix(1)-0.5)*yratio + 0.5, FORMAT='(E14.7)'

; Scale either the CDELT parameters or the CD1_1 parameters.

 if (noparams EQ 0) or ( noparams EQ 1) then begin 

    cdelt = astr.cdelt
    sxaddpar, newhd, 'CDELT1', CDELT(0)/xratio
    sxaddpar, newhd, 'CDELT2', CDELT(1)/yratio

 endif else if noparams EQ 2 then begin

    cd = astr.cd
    sxaddpar, newhd, 'CD1_1', cd(0,0)/xratio, FORMAT='(E14.7)'
    sxaddpar, newhd, 'CD1_2', cd(0,1)/yratio, FORMAT='(E14.7)'
    sxaddpar, newhd, 'CD2_1', cd(1,0)/xratio, FORMAT='(E14.7)'
    sxaddpar, newhd, 'CD2_2', cd(1,1)/yratio, FORMAT='(E14.7)'

 endif 
 endif

; Adjust BZERO and BSCALE for new pixel size

 bscale = sxpar( oldhd, 'BSCALE')
 if (bscale NE 0) and (bscale NE 1) then $
    sxaddpar, newhd, 'BSCALE', bscale/pix_ratio, 'Calibration Factor'
 bzero = sxpar( oldhd, 'BZERO')
    if (bzero NE 0) then sxaddpar, newhd, 'BZERO', bzero/pix_ratio, $
       ' Additive Constant for Calibration'

 pixelsiz = sxpar( oldhd,'PIXELSIZ' )
 if !ERR NE -1 then sxaddpar, newhd, 'PIXELSIZ', pixelsiz/xratio

 if npar EQ 2 then oldhd = newhd else $
    if npar EQ 1 then oldim = newhd

 return
 end
